#ifndef CLIST_H
#define CLIST_H

/*
 * clist: A double linked list. Initialize first item to NULL.
 */

#include <features.h>

__BEGIN_DECLS

#define CCOMPARE_FUNC(x) int (* x )(void *a, void *b)
#define CITEM_FUNC(x) void (* x )(void *data, void *user_data)
#define CITEM_FREE(x) void (* x )(void *data)

#define clist_previous(x) { x = x->prev; }
#define clist_next(x) { x = x->next; }


typedef struct clist_t {
    void *data;
    struct clist_t *next;
    struct clist_t *prev;
} clist;


/* function definitions */
inline clist *clist_alloc();
inline void clist_free_1(clist *list);
clist *clist_append(clist *list, void *data);
clist *clist_prepend(clist *list, void *data);
clist *clist_insert(clist *list, void *data, int pos);
clist *clist_insert_sorted(clist *list, void *data, CCOMPARE_FUNC(ccfunc));
clist *clist_remove(clist *list, void *data);
clist *clist_remove_custfree(clist *list, void *data, CITEM_FREE(cfree));
clist *clist_remove_link(clist *list, clist *link);
clist *clist_free(clist *list);
clist *clist_custfree(clist *list, CITEM_FREE(cfree));
int clist_length(clist *list);
clist *clist_copy(clist *list);
clist *clist_reverse(clist *list);
clist *clist_sort(clist *list, CCOMPARE_FUNC(ccfunc));
clist *clist_concat(clist *list1, clist *list2);
void clist_foreach(clist *list, CITEM_FUNC(cifunc), void *user_data);
clist *clist_first(clist *list);
clist *clist_last(clist *list);
clist *clist_nth(clist *list, int n);
void *clist_nth_data(clist *list, int n);
clist *clist_find(clist *list, void *data);
/* ccfunc should return 0 when the item is found */
clist *clist_find_custom(clist *list, void *user_data, CCOMPARE_FUNC(ccfunc));
int clist_position(clist *list, clist *find);
int clist_index(clist *list, void *data);


__END_DECLS

#endif /* CLIST_H */
