#ifndef CONF_H
#define CONF_H

#include <features.h>
__BEGIN_DECLS

#include "sock.h"
#include "msgs.h"
#include "clist.h"

/* Comment character in the config files */
#define COMMENT_CHAR '#'

/* Tag types supported so far */
#define TAGTYPE_STR	0
#define TAGTYPE_QOTDSTR	1
#define TAGTYPE_INT	2
#define TAGTYPE_FLOAT	3
#define TAGTYPE_NADDR	4
#define TAGTYPE_NULL	5

/* Character set for tags */
#define TAGCSET_NULL	""
#define TAGCSET_ALL	""
#define TAGCSET_INT	""
#define TAGCSET_FLOAT	""
#define TAGCSET_ALPHA	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
#define TAGCSET_ALNUM	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"

/* This struct initally stores the description of the tags
 * to be read from the config file. After reading, the value
 * of the tags are also stored within this struct. */
typedef struct {
	char *tag;		/* tag name */
	char *charset;		/* character set allowed */
	int type;		/* type of value expected */
	int multiple;		/* allow multiple tags of this type? */
	clist *values;		/* actual value after read */
} taginfo_t;


/* Function prototypes */
int conf_read(char *cfpath, taginfo_t *tags);
int conf_free(taginfo_t tags[]);

/* Function declaration for a tag format handler */
#define CONFTAG_HANDLER(x) int x(taginfo_t *tinf, char *targ)

/* Message handlers for taginfo_t.type */
typedef struct {
	        int (*func)(taginfo_t *, char *);
} conftag_handler_t;

extern conftag_handler_t conftag_handlers[];

/* The total number of tag handlers */
#define NUMBER_OF_TAGHANDLERS (sizeof(conftag_handlers)/sizeof(conftag_handler_t))

__END_DECLS

#endif /* CONF_H */
