#ifndef FILEMAN_H
#define FILEMAN_H

#include <features.h>
__BEGIN_DECLS

#include "structs.h"


#define FTP_HND_EVENT(x) void x(ftpinfo_t *ftpitem, void *data)
#define FTP_HND_EVENT_PTR(x) void (*x)(struct _ftpinfo_t *ftpitem, void *data)

/* Block size of FTP send */
#define FTP_SEND_BLOCK_SIZE 4096

/* Time in secs, after which the transfer will be aborted */
#define FTP_TIMEOUT_INTERVAL 120

/* File permissions to use for newly created files */
#define FTP_DEFAULT_PERM (S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)

/* Values for ftpinfo_t.status */
#define FTPSTAT_PENDING 1
#define FTPSTAT_COMPLETE 2
#define FTPSTAT_TIMEOUT 3
#define FTPSTAT_CANCEL 4
#define FTPSTAT_FAIL 5


/* The file manager is responsible for freeing 'user_data' and
 * 'filename'. Only the user_data must be allocated by the caller. */
typedef struct _ftpinfo_t {
	ftpid_t ftp_id;		/* identifier of this transfer */
	daemon_id remote_dem;	/* send/recv to/from daemon */
	int status;		/* current status of transfer */
	char *filename;		/* without path */
	unsigned long size;	/* total size of the file */
	unsigned long cur_size;	/* size currently transferred */
	int blks_trans;		/* blocks transferred uptill now */
	time_t last;		/* time of last activity */
	int fd;			/* open handle of file */
	
	/* the following handler is called on ftp timeout, cancel,
	 * and file transfer completion. The 'ftpitem->status' field
	 * is set accordingly before calling the handler. */
	void *user_data;	/* dynamically allocated data */
	FTP_HND_EVENT_PTR(hnd_event);
} ftpinfo_t;


/* Function definitions */
void ftp_item_free(ftpinfo_t *fitem);
int ftp_item_cmp(void *a, void *b);
FTP_HND_EVENT(ftp_default_recv_event);
FTP_HND_EVENT(ftp_default_send_event);

int ftp_set_dl_path(char *path);
ftpinfo_t *ftp_get_send(ftpid_t *fid);
ftpinfo_t *ftp_get_recv(ftpid_t *fid);
int ftp_remove_send(ftpid_t *fid);
int ftp_remove_recv(ftpid_t *fid);
int ftp_cancel_send(ftpid_t *fid);
int ftp_cancel_recv(ftpid_t *fid);
int ftp_send_block(ftpid_t *fid, int fd);
int ftp_recv_block(ftpid_t *fid, void *data, unsigned long size);

ftpid_t *ftp_send(daemon_id *to, char *filename,
		FTP_HND_EVENT_PTR(fhnd), void *user_data);
int ftp_recv(ftpid_t *from, char *filename, unsigned long size, char *md5sum,
		FTP_HND_EVENT_PTR(fhnd), void *user_data);

void ftp_run();

extern char *ftp_dl_path;

__END_DECLS

#endif /* FILEMAN_H */
