#ifndef LIBNC_H
#define LIBNC_H

#include <features.h>
__BEGIN_DECLS

#include "msghnd.h"

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/* Maximum size of all RPC parameters combined */
#define MAX_RPCARG_SIZE 512500

/* Start symbol for the master task */
#define TASK_MASTER_STARTSYM_STR "task_main"
#define TASK_MASTER_STARTSYM task_main

/* Functions called by nc_call and nc_accept */
#define RPC_FUNC(x)

/* Task definitions of functions used by nc_createtask */
#define SUBTASK(x) void x()
#define SUBTASK_PTR(x) void (*x)()

#define TOTAL_RPCS (sizeof(nc_rpcs)/sizeof(nc_rpcdef_t))
#define TOTAL_TASKS (sizeof(nc_tasks)/sizeof(nc_taskdef_t))

/* File descriptor used for msg communication with daemon */
#define FD_MSG 3

/*
 * User function definition list. This list maps the user function
 * IDs used by the user onto the parameter types and return value
 * of the user functions. This would in the future be generated by
 * a netcluster preprocessor. The user functions are actually the
 * transactions supported by the user processes.
 * - parameter list: printf style format of the parameters.
 *   The following format specifiers are currently recognised:
 *     %d: A 32 bit integer, signed or unsigned.
 *     %c: An 8 bit character, signed or unsigned.
 *     %s: NULL terminated ASCII string of any length.
 *     %t: A task_id type. Must be given in the parameters by reference.
 * - return value: The format specifiers are the same as the
 *   parameter list with one added specified "%z", which means a void
 *   return type. Only a single specifier is accepted here.
 */
typedef struct {
	char *func_name;
	char *fmt_param;
	char *fmt_ret;
} nc_rpcdef_t;

/*
 * User task definition list. This is the symbol table of the tasks
 * defined by the user. Each task may support the above user functions
 * which are actually the transactions.
 */
typedef struct {
	char *task_name;
	SUBTASK_PTR(task_sym);
} nc_taskdef_t;

extern nc_rpcdef_t nc_rpcs[];
extern nc_taskdef_t nc_tasks[];

#define msg_write(xmsg, xdata) msg_fd_write(xmsg, xdata, FD_MSG)
#define msg_read(xmsg, xdata) msg_fd_read(xmsg, xdata, FD_MSG)

/* Function definitions */
int msg_fd_write(msghdr_t *msg, void *data, int fd);
int read_nbytes(int fd, char *buffer, int n);
int msg_fd_read(msghdr_t *hdr, void *data, int fd);

/* Highlevel libnc interface */
task_id nc_myid();
task_id nc_createtask(char *start_sym);
int nc_call(task_id *tid, u_int32_t funcid, ...);
int nc_accept(u_int32_t funcid, ...);
int nc_accept_noblock(u_int32_t funcid, ...);
int nc_treturn(int dummy, ...);

__END_DECLS

#endif /* LIBNC_H */
