#ifndef MSGHND_H
#define MSGHND_H

#include <features.h>
__BEGIN_DECLS

#include <sys/types.h>
#include "structs.h"
#include "clist.h"
#include "msghnd.h"


/* Message data. To access, pass a pointer to start of msg. */
#define MSG_DATA(x) ((char*)x + sizeof(msghdr_t))
#define MSG_SIZE(x) (sizeof(msghdr_t) + x->data_size)

/* Message class by source and dest given msghdr_t */
#if 0
#define MSGCLASS_TASK(msg) (((msg).msg_type >= 10) && ((msg).data_size >= sizeof(task_id)))
#define MSGCLASS_DAEMON(msg) ((msg).msg_type < 10)
#endif

/* COMMON-TO-ALL MESSAGES */
#define MSG_INVALID			0
#define MSG_PING			1
#define MSG_PONG			2
#define MSG_SYSLOG			3

/* SUBSYSTEM VAS MESSAGES */
#define MSG_VAS_PONG			20
#define MSG_VAS_REG			21
#define MSG_VAS_UNREG			22 
#define MSG_VAS_ISREG			23
#define MSG_VAS_SETLOAD			24
#define MSG_VAS_GETLOAD			25
#define MSG_VAS_GETMINLOAD		26
#define MSG_VAS_DISPLAY			27

/* SUBSYSTEM DAEMON MESSAGES */
/* Communication with VAS */
#define MSG_VAS_PING			30
#define MSG_VAS_REG_RPLY		31
#define MSG_VAS_ISREG_RPLY		32
#define MSG_VAS_GETLOAD_RPLY		33
#define MSG_VAS_GETMINLOAD_RPLY		34
/* Task management */
#define MSG_CREATE_MASTER		35
#define MSG_CREATE_TASK			36
#define MSG_CREATE_LOCALTASK		37
#define MSG_CREATE_TASK_RPLY		38
#define MSG_IOREDIR_OUT			39
#define MSG_ISTERMINATE			40
#define MSG_NOTERMINATE			41
#define MSG_DOTERMINATE			42
/* File transfer */
#define MSG_FTP_INIT			46
#define MSG_FTP_REQ			47
#define MSG_FTP_DATA			48
#define MSG_FTP_DEINIT			49
/* libnc messages */
#define MSG_RPC_CALL			50
#define MSG_RPC_RPLY			51
#define MSG_RPC_ACCEPT			52
#define MSG_MY_TASKID			53
#define MSG_TERMINATE			54
#define MSG_ABORT			55


/* Struct for storing the msg header. The data directly follows the
 * header. Some optimization have been used:
 * - sender=SELF_NADDR. If the ip and port are equal to our own addr
 *   for the 'sender' field of an incoming message, then the message
 *   is considered as LOCAL. It is not handled by the connection
 *   manager. Instead it's pipe is directly fd_open'ed for the
 *   incoming messages. Such a message must be MSGCLASS_TASK.
 * - receiver=SELF_NADDR. If the receiver's address is the same as
 *   our own address for outgoing messages, then the message is
 *   considered LOCAL. Such a message must be MSGCLASS_TASK.
 * - The data of every MSGCLASS_TASK contains a task_id as the first
 *   item. Therefore, the size of the msg cant be zero. */
typedef struct {
	daemon_id sender;
	daemon_id receiver;
	u_int16_t msg_type;	/* type in network byte order */
	u_int16_t priority;	/* in network byte order */
	u_int32_t data_size;	/* in network byte order */

	/* The following data is for transactions only */
	u_int16_t trans_id;	/* transaction id */
	u_char trans_isrply;	/* is a transaction reply? TRUE or FALSE */

	/* void *data; for the msg data following this header */
} msghdr_t;

__END_DECLS

#endif /* MSGHND_H */
