#ifndef MSGS_H
#define MSGS_H

#include <features.h>
__BEGIN_DECLS

#include <sys/types.h>
#include "structs.h"
#include "clist.h"
#include "msghnd.h"


/* Message class for msginfo_t.mclass */
#define MSGCLASS_LOCAL	0
#define MSGCLASS_REMOTE	1

/* Message status for msginfo_t.status */
#define MSGSTAT_DEL 0
#define MSGSTAT_UNDELIV 1
#define MSGSTAT_WAIT 2
#define MSGSTAT_WAITTTL 3
#define MSGSTAT_EXPIRED 4

/* Data type used to store msgs after receival */
typedef struct {
	msghdr_t *hdr;		/* Hdr followed by data: must be free'd */
	void *data;		/* Msg data pointing after above hdr:
				 *   must not be free'd explicitly */
	int ttl;		/* Local time to live of message */
	int fd;			/* fd from which this msg arrived */
	int mclass;		/* Class local or remote */
	int status;		/* Current status of msg */
} msginfo_t;


/* Function definitions */
int msg_free_deletable();
int msg_enqueue(msghdr_t *msg, int fd, int mclass);
int msg_dispatch();


/* Function declaration for a msg handler */
#define MSG_HANDLER(x) int x(msghdr_t *hdr, void *data, int fd, transact_t *trans)

/* Message handlers for msghdr_t.msg_type */
typedef struct {
	int (*func)(msghdr_t *, void *, int, transact_t *);
} msghandler_t;

/* The total number of message handlers */

#define NUMBER_OF_HANDLERS 60


extern msghandler_t msg_handlers[NUMBER_OF_HANDLERS];
extern clist *in_msgs;

__END_DECLS

#endif /* MSGS_H */
