#ifndef TASK_H
#define TASK_H

#include <features.h>
__BEGIN_DECLS

#include <sys/types.h>
#include "structs.h"


/* Start symbol for master tasks */
#define TASK_MASTER_STARTSYM "task_main"

/* Possible (internal) states of tasks */
#define TSTATE_NOTFOUND 0
#define TSTATE_EXECUTE 1
#define TSTATE_COMPLETE 2
#define TSTATE_WAIT 3
#define TSTATE_SUSPEND 4
#define TSTATE_TERMINATE 5
#define TSTATE_UNDETERMINED 6


/* Stored a list of the children of a parent task. Is used in
 * the taskinfo_t struct. The list is terminated by a NULL. */
typedef struct _childinfo_t {
	task_id child;
	struct _childinfo_t *next;
} childinfo_t;

/* Info of all the tasks running on the machine */
typedef struct _taskinfo_t {
	task_id id;			/* current task's id */
	task_id parent;			/* parent task's id */
	task_id master;			/* master task's id */
	int priority;			/* current priority of the task */
	char *sym_name;			/* symbolic name of process */
	char *start_sym;		/* start symbol of process */
	char *file_name;		/* path to file name */
	int state;			/* current internal state */
	void *state_data;		/* see state data types below */
	int loader_fd;			/* comm fd with the loader */
	int msg_io[2];			/* pipe for reading msgs */
	int std_io[2];			/* piped std out/err/in of task */

	childinfo_t *child_list;	/* list of direct children of task */
	struct _taskinfo_t* next;	/* next task - linked list */
} taskinfo_t;

extern taskinfo_t *task_list;


/* Functions for task list managing */
int task_free_list();
int task_add( taskinfo_t *new_task );
int task_remove( task_id *tid );
taskinfo_t *task_find( task_id *tid );
taskinfo_t *task_find_name( char *name );
taskinfo_t *task_find_stdio( int fd );
taskinfo_t *task_find_msgio( int fd );
taskinfo_t *task_find_child(task_id *child);
int task_all_terminated(task_id *master);
int task_remove_terminated( task_id *master );
int task_add_child(task_id *parent, task_id *child);
int task_remove_child(task_id *parent, task_id *child);
int task_remove_all_child(task_id *parent);
void task_display_all();

/* Function for task management */
int task_get_msgio(task_id *tid);
int task_inc_load();
int task_dec_load();
int task_calc_load();
int task_update_load();
int task_terminate(task_id *tid, u_int32_t status);
childinfo_t *task_remote_child(task_id *master);
childinfo_t *task_abort(task_id *master);
task_id *create_master(char *binary, int priority, int argc, char *argvs,
		int loader_fd);
task_id *create_local_task(char *binary, char *start_sym, task_id *master,
		task_id *parent, int priority);

int create_task_minload(taskreq_t *treq);
int create_task_ftp(taskreq_t *treq, daemon_id *min_loaded);

/* Data and functions for task states */

/* TSTATE_WAIT state data */
typedef struct {
	u_int16_t msg_type;
	void *data;
} tstate_wait_t;

int task_state_datafree(taskinfo_t *tinf);
int task_setstate_wait(taskinfo_t *tinf, tstate_wait_t *sdata);
int task_setstate_execute(taskinfo_t *tinf);
int task_setstate_terminate(taskinfo_t *tinf, u_int32_t status);


__END_DECLS

#endif /* TASK_H */
