/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Interface to the screen
 *
 */
#ifndef SCREEN_H
#define SCREEN_H

#include "data.h"

#if defined(X_UNIX) || defined(X_DJGPP)
#include <curses.h>
#else
#include <conio.h>
#endif


#ifdef X_BORLAND
#define COLOR_BLACK   0
#define COLOR_RED     6
#define COLOR_GREEN   2
#define COLOR_YELLOW  4
#define COLOR_BLUE    1
#define COLOR_MAGENTA 3
#define COLOR_CYAN    5
#define COLOR_WHITE   7

#define A_BOLD        128
#endif


/* colors for the current theme */
#define H_INTENSITY 64


/* A subscreen class. Used to create subwindows */
class CSubScreen {
    friend class CScreen;
public:
    CSubScreen(int,int,int,int);
    ~CSubScreen();

private:
    int x1,y1,x2,y2;

#if defined(X_UNIX) || defined(X_DJGPP)
    WINDOW *Data;
#else
    char *Data;
#endif
};



/* The main screen class */
class CScreen {
public:
    CScreen();
    ~CScreen();

    int GetMaxX();
    int GetMaxY();

    CSubScreen *SaveScreen(int,int,int,int);
    void RestoreScreen(CSubScreen *,int,int);
    void RestoreScreen(CSubScreen *);

    void Clear();
    void Clear(int);
    void Clear(int,int,int,int);

    void Fill(int,int,int,int,int);

    void GetCursor(int &,int &);
    void SetCursor(int,int);

    void SetColor(int = COLOR_WHITE,int = COLOR_BLACK);

    void PutChar(int);
    void PutCharXY(int,int,int);

    void PrintXY(int,int,char *);
    int (*Print)(char *,...);

    void Refresh();

    static int IsInit();
private:
    int max_x;
    int max_y;

#if defined(X_UNIX) || defined(X_DJGPP)
    int ColorIndex[8][8];
#endif

    static int Init;

    /* Used to init the curses color pairs */
    void InitColorPairs();
};


extern CScreen MyScreen;

extern int MAX_X;
extern int MAX_Y;


#endif /* SCREEN_H */
