/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * The main control class and its support classes
 *
 */
#ifndef XCONTROL_H
#define XCONTROL_H

#include "data.h"
#include "xbase.h"
#include "mouse.h"
#include "keyboard.h"
#include "contain.h"
#include "xinter.h"
#include "xdesktop.h"


/* Control Events */
#define RE_KEYDOWN    1
#define RE_LEFTDOWN   2
#define RE_RIGHTDOWN  3
#define RE_LEFTUP     4
#define RE_RIGHTUP    5
#define RE_MOVE       6


/* A message item class */
class CMessageItem {
public:
    int Type;
    CPoint p;
    int Key;
    int Status;
};

/* The message container */
class CMessage {
public:
    CMessage();

    bool Add();
    bool Remove();
 
    CMessageItem Message;

public:
    CContainer List;
};

/* The main control of the whole interface */
class CControl {
public:
    CControl();

    bool SetDesktop(char *);

    /* Managing windows */
    bool CreateWindow(CWindow *);
    bool CloseWindow();
    bool HideWindow();
    bool CloseAll();
    bool SwitchWindow(int);
    CWindow *GetActiveWindow();

    bool RepaintAll();
    bool Run();

    /* Exit and Init */
    bool Init();
    bool Exit();
private:
    /* Utility functions */
    void GetEvent();
    void DispatchEvent();
    
    bool ControlFromPoint(CPoint);


    /* Data items for storing messages, windows etc */
    CMessage MessageQueue;
    CContainer WindowList;

    CWindow *ActiveWindow;

    CDeskTop Desktop;

public:
    /* The mouse and keyboard classes. Used to get events */
    CMouse Mouse;
    CKeyboard Keyboard;
};

extern CControl Control;

#endif /* XCONTROL_H */
