/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * All the interface controls classes
 *
 */

#ifndef XINTER_H
#define XINTER_H

#define CSIG_ONCLOSE 1000
#define CSIG_FILEDIALOG 1001


#include "data.h"
#include "xbase.h"
#include "screen.h"
#include "contain.h"


/* An idle and key handler */
void re_idle_handler();
bool re_key_handler(int Key, int Status);


/* The base class - events */
class CEvent {
public:
    CEvent();
    virtual ~CEvent();

    /* All the events */
    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status);
    virtual bool OnSignal(int Handle,long int Event, void *Data[5]);
};


/* Window style codes */
#define CWIN_CLOSEB    1
#define CWIN_BORDER    2
#define CWIN_RESIZE    4
#define CWIN_MOVE      8
#define CWIN_TITLE     16
#define CWIN_DIALOG    32
#define CWIN_NOCONTROL 64
#define CWIN_TABCYCLE  128
#define CWIN_NOTABSTOP 256


/* Window colors */
#define CWIN_TITLECOL   THEME_NORM
#define CWIN_CROSSCOL   THEME_NORM
#define CWIN_DTITLECOL  THEME_DARK
#define CWIN_DCROSSCOL  THEME_NORM

/* Special keys for this class */
#define CWIN_KEY_CYCLE  9

class CWindow;

/* The window property class */
class CWinProperty {
public:
    CWinProperty();
    ~CWinProperty();

    void SetTitle(char *s);
    void SetStyle(long int);
    void SetCoords(int,int,int,int);


    long int Style;
    char *Title;
    CRect Rect;

    CWindow *Parent;
    CWindow *Response;

    int Handle;
    bool Visible;
};


/* The Window class */
class CWindow : public CEvent {
public:
    CWindow();
    virtual ~CWindow();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status);
    virtual bool OnSignal(int Handle,long int Event, void *Data[5]);


    CWinProperty Property;

    CRect MapToScreen(CRect,CWindow *);

    int AddControl(CWindow *);
    bool DeleteControl(int Handle);
    CWindow *GetControl(int Handle);
    bool NextControl();


    int ControlFromPoint(CPoint);

protected:
    CContainer ControlList;
    int ControlCount;
    int CurrentControl;

    int IsCreated;
    int IsMoving;
    int IsResizing;

private:
    /* Variables for moving the window */
    int mov_down_x, mov_down_y;

    /* And for resizing */
    int res_down_x, res_down_y;

};


/* The label colors */
#define CLABEL_COLOR2 THEME_NORM
#define CLABEL_COLOR1 THEME_LIGHT

/* The label class */
class CLabel : public CWindow {
public:
    CLabel();
    CLabel(char *);
    virtual ~CLabel();

    void SetLabel(char *);
    char *GetLabel();

    virtual bool OnPreCreate();
    virtual bool OnClose();
    virtual bool OnPaint();
    virtual bool OnGetFocus();
    
private:
    /* Store the label text */
    char *Text;
};


/* Editbox colors */
#define CEDIT_ENDCOL1 THEME_NORM
#define CEDIT_ENDCOL2 THEME_DARK
#define CEDIT_TEXTCOL THEME_NORM

/* Editbox char sets */
#define CEDIT_ALPHANUM "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\
                        1234567890()@!.,'[]-_#"
#define CEDIT_NUM "1234567890-"
#define CEDIT_FLOAT "1234567890.-"
#define CEDIT_ALPHA "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ "

/* The Editbox class */
class CEditBox : public CWindow {
public:
    CEditBox();
    CEditBox(const CEditBox &);
    virtual ~CEditBox();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);


    bool GetText(char *);
    bool SetText(char *);

    bool SetLength(int);
    int GetLength();

    bool SetCharSet(char *);

private:
    /* Data for the editbox text and charset */
    char *Buffer;
    char *CharSet;

    /* Cursor and scroll positions */
    int Cursor;
    int Scroll;
    int Length;

    /* Utility functions */
    void Delch(int);
    void Insch(int);
};


/* Checkbox colors */
#define CCHECKBOX_UNSELCOL THEME_DARK
#define CCHECKBOX_SELCOL   THEME_LIGHT
#define CCHECKBOX_TEXTCOL1 THEME_LIGHT
#define CCHECKBOX_TEXTCOL2 THEME_NORM

/* The checkbox class */
class CCheckBox: public CWindow {
public:
    CCheckBox();
    CCheckBox(const CCheckBox&);
    virtual ~CCheckBox();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    bool GetState();
    void SetState(int);

    void SetText(char *);
    
private:
    /* State and the label of the checkbox */
    bool State;
    char *Text;
};


/* Button colors */
#define CBUTTON_TNORM THEME_NORM
#define CBUTTON_THIGH THEME_LIGHT
#define CBUTTON_LNORM THEME_DARK
#define CBUTTON_LHIGH THEME_NORM

/* The button class */
class CButton : public CWindow {
public:
    CButton();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    void Enable();
    void Disable();

private:
    /* Button State */
    bool Enabled;
    int MouseOver;
};


/* Scrollbar colors */
#define CSCROLL_PCOL THEME_NORM
#define CSCROLL_BCOL THEME_DARK

/* The scrollbar class */
class CScrollBar : public CWindow {
public:
    CScrollBar();
    virtual ~CScrollBar();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);


    void SetRange(int,int,int);
    int GetCurrent();
    void SetCurrent(int);

    bool ShowFocus(bool);

private:
    /* Scrollbar limits */
    int Min,Max,Current;
    bool showFocus;
};


/* Listbox colors */
#define CLISTBOX_TNORMCOL THEME_DARK
#define CLISTBOX_THIGHCOL THEME_NORM

#define CLISTBOX_DELKEY KEY_DC
#define CLISTBOX_REORDER_BW '<'
#define CLISTBOX_REORDER_FW '>'

/* The listbox class */
class CListBox : public CWindow {
public:
    CListBox();
    virtual ~CListBox();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status);


    /* Functions to set different styles */
    void SetScrollable(bool);
    void SetFollow(bool);
    void SetRemovable(bool);
    void SetReorder(bool);

    /* Adding and del items to the listbox */
    bool AddItem(char *);
    bool DeleteAll();

    /* Item selecttion */
    void SetSelect(int);
    int GetSelect();
    char *GetItem();
    char *GetItem(int n);
    CContainer *GetItemContainer();

    int GetTotal();
protected:
    /* The scrollbar data members */
    CContainer Items;
    
    int Select;
    int Scroll;

    bool FollowMode;
    bool Scrollable;
    bool RemoveMode;
    bool ReorderMode;

    int ScrollHandle;
};


/* The radiobox colors */
#define CRADIO_BCOL  THEME_DARK
#define CRADIO_TCOL1 THEME_LIGHT
#define CRADIO_TCOL2 THEME_NORM
#define CRADIO_SCOL  THEME_LIGHT

/* The radiobox class */
class CRadio : public CWindow {
public:
    CRadio();
    virtual ~CRadio();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);


    /* Setting and getting the radiobox state */
    bool SetSelect(int);
    int GetSelect();

    /* adding items to the radiobox */
    bool AddItem(char *);
    char *GetItem(int);

    bool DeleteAll();

private:
    int Select;
    CContainer Items;
};


/* Slider colors */
#define CSLIDER_HLINE THEME_DARK
#define CSLIDER_SCOL THEME_LIGHT
#define CSLIDER_USCOL THEME_NORM
#define CSLIDER_ACOL THEME_NORM

/* The slider class */
class CSlider : public CWindow {
public:
    CSlider();
    virtual ~CSlider();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);


    /* Setting the slider range and getting current status */
    void SetRange(int,int,int);
    int GetCurrent();
    void SetCurrent(int);

private:
    /* Slider limits */
    int Min,Max,Current;
};



/* Progress bar colors */
#define CPROGRESS_SEL THEME_NORM
#define CPROGRESS_UNSEL THEME_DARK

/* The progressbar class */
class CProgress : public CWindow {
public:
    CProgress();
    virtual ~CProgress();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);


    /* Setting and getting the range of the slider */
    void SetRange(int,int,int);
    int GetCurrent();
    void SetCurrent(int);

private:
    /* The limits of the slider */
    int Min,Max,Current;
};


/* The text area colors and charset */
#define CTEXTAREA_COL THEME_NORM
#define CTEXTAREA_ALPHANUM "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQ\
                           RSTUVWXYZ1234567890,$[]@#-:"

/* The textarea class */
class CTextArea : public CWindow {
public:
    CTextArea();
    virtual ~CTextArea();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status);


    /* Setting the char set */
    bool SetCharSet(char *);
    
    /* Getting info about the text area */
    int TotalLines();
    int GetLength();

    bool SetMaxLines(int);

    /* Setting data in the textarea */
    bool GetLine(int,char *);
    bool SetLine(int,char *);
    bool AddLine(char *);

    bool IsEmpty(int);

    bool Clear();

private:
    /* Some utility functions */
    bool CreateLine();
    bool AddScroll();
    void Delch(int);
    void Insch();

    CContainer Data;

    /* Current state of the textarea */
    int CurX,CurY;
    int Length;
    int MaxLines;

    char *CharSet;

    int Scroll;
    bool Scrollable;
    int ScrollHandle;
};


/* Textbox colors */
#define CTEXTBOX_COL THEME_NORM

/* The textbox class */
class CTextBox : public CWindow {
public:
    CTextBox();
    virtual ~CTextBox();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status);


    /* Setting and getting text from the textbox */
    bool SetText(char *);
    bool CatText(char *);
    bool ReAlign();

    char *GetLine(int);
    int TotalLines();

private:
    /* Utility function to perform a wordwrap */
    int WordLength(char *);

    char *RawData;
    CContainer Data;

    int Scroll;
    bool Scrollable;
    int ScrollHandle;
};



/* The menu colors */
#define CMENU_UNSELCOL THEME_NORM
#define CMENU_SELCOL THEME_LIGHT
#define CMENU_BCOL THEME_DARK

/* A single menu node */
class CMenuNode {
    friend class CMenu;
public:
    CMenuNode(char *,int);
    ~CMenuNode();
private:
    /* The text displayed in the top bar */
    char *Text;
    int Handle;
};

/* The menu class */
class CMenu : public CWindow {
public:
    CMenu();
    virtual ~CMenu();

    virtual bool OnCreate();
    virtual bool OnPreCreate();
    virtual bool OnClose();

    virtual bool OnPaint();
    virtual bool OnGetFocus();
    virtual bool OnLoseFocus();

    virtual bool OnKeyDown(int Key,int Status);

    virtual bool OnLeftDown(CPoint p,int Status);
    virtual bool OnRightDown(CPoint p,int Status);
    virtual bool OnLeftUp(CPoint p,int Status);
    virtual bool OnRightUp(CPoint p,int Status);
    virtual bool OnMove(CPoint p,int Status);

    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status);


    /* Add items to the menu. Dynamically */
    int AddItem(char *,CListBox *);

private:
    CContainer List;
    int Select;
};

#endif /* XINTER_H */
