/*
 * Generic Container Class
 * Written by Fakhir Ansari <ansari89@yahoo.com
 *
 * Module - The container class
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "contain.h"


CContainer::CContainer()
{
    IsInit = false;
}

CContainer::CContainer(int Len)
{
    Initialize(Len);
}

void CContainer::Initialize(int Len)
{
    Length = Len;
    Total = 0;
    IsInit = true;
    
    Top=Bottom=Current=NULL;
}

CContainer::CItem::CItem()
{
    Next=Prev=NULL;
    Buffer=NULL;
}


bool CContainer::AddTop(void *data)
{
    CItem *Points = new CItem;
    
    if (!Points || !IsInit) 
	ListError();

    Points->Buffer = new char[Length];
    if (!Points->Buffer) {
	delete Points;
	ListError();
    }

    memcpy(Points->Buffer,data,Length);

    if (!Top)
	Top=Bottom=Current=Points;
    else {
	Top->Next = Points;
	Points->Prev = Top;
	Top = Points;
    }

    Total++;

    return true;
}

bool CContainer::AddBottom(void *data)
{
    CItem *Points = new CItem;
    
    if (!Points || !IsInit)  
	ListError();

    Points->Buffer = new char[Length];
    if (!Points->Buffer) {
	delete Points;
	ListError();
    }

    memcpy(Points->Buffer,data,Length);

    if (!Top)
	Top=Bottom=Current=Points;
    else {
	Bottom->Prev = Points;
	Points->Next = Bottom;
	Bottom = Points;
    }

    Total++;

    return true;
}

bool CContainer::InsertItem(int n,void *data)
{
    if (n == 0 || Total == 0)
	return AddBottom(data);
    else if (n == Total)
	return AddTop(data);

    if (n<0 || n> Total)
	return false;
    else if (!IsInit)
	ListError();


    CItem *Points = new CItem;
    CItem *Index = Bottom;
    int i;
    
    if (!Points) 
	ListError();

    Points->Buffer = new char[Length];
    if (!Points->Buffer) {
	delete Points;
	ListError();
    }

    memcpy(Points->Buffer,data,Length);


    for (i=0;i<n;i++)
	Index = Index->Next;

    Points->Prev = Index->Prev;
    Points->Next = Index;
    Index->Prev->Next = Points;
    Index->Prev = Points;

    Total++;

    return true;
}


void *CContainer::GetTop()
{
    if (!Top)
	return NULL;

    return Top->Buffer;
}

void *CContainer::GetBottom()
{
    if (!Bottom)
	return NULL;

    return Bottom->Buffer;
}

void *CContainer::GetItem(int pos)
{
    int i;

    if (!Top)
	return NULL;
    else if (pos < 0 || pos >= Total)
	return NULL;

    CItem *Points = Bottom;

    for (i=0;i<pos;i++) {
	Points = Points->Next;
    }

    return Points->Buffer;
}

bool CContainer::RemoveTop()
{
    CItem *OldOne;

    if (!Top || Total==0)
	return false;


    OldOne = Top;
    Top = Top->Prev;
    if (Top) {
	Top->Next = NULL;
    }

    delete [] OldOne->Buffer;
    delete OldOne;
    Total--;

    if (Total == 0)
	Top=Bottom=Current=NULL;

    return true;
}

bool CContainer::RemoveBottom()
{
    CItem *OldOne;

    if (!Bottom || !Top || Total==0)
	return false;


    if (Total == 1) {
	delete [] Bottom->Buffer;
	delete Bottom;

	Top=Bottom=Current=NULL;
	Total=0;
    }
    else {
	OldOne = Bottom;
	Bottom = Bottom->Next;
	if (Bottom && Total!=1)
	    Bottom->Prev = NULL;
	
	delete [] OldOne->Buffer;
	delete OldOne;
	Total--;
    }

    return true;
}

bool CContainer::Reset()
{
    Current = Bottom;
    return true;
}

void *CContainer::GetCurrent()
{
    if (!Current)
	return NULL;
    else
	return Current->Buffer;
}

bool CContainer::MoveNext()
{
    if (!Current)
	return false;

    Current = Current->Next;
    return true;
}

bool CContainer::MovePrev()
{
    if (!Current)
	return false;

    Current = Current->Prev;
    return true;

}

int CContainer::GetTotal()
{
    return Total;
}

int CContainer::IsEmpty()
{
    return !(Total);
}

void CContainer::ListError()
{
    printf("Allocation or list Error: Aborting!\n");
    exit(EXIT_FAILURE);
}

bool CContainer::DeleteAll()
{
    int i;

    if (!Top || !Bottom)
	return false;


    CItem *Points = Bottom;

    for (i=0;i<Total;i++) {
	Current = Points->Next;

	delete [] Points->Buffer;
	delete Points;

	Points = Current;
    }

    Top=Current=Bottom=NULL;
    Total = 0;

    return true;
}

bool CContainer::Delete(int Item)
{
    int i;
    CItem *Points;

    if (!Top || !Bottom)
	return false;
    else if (Item >= Total || Item < 0)
	return false;

    if (Item == 0) {
	RemoveBottom();
    }
    else if (Item == Total-1) {
	RemoveTop();
    }
    else {
	Points = Bottom;
	for (i=0;i<Item;i++) {
	    Points = Points->Next;
	}

	if (Points->Prev)
	    Points->Prev->Next = Points->Next;
	if (Points->Next)
	    Points->Next->Prev = Points->Prev;

	delete [] Points->Buffer;
	delete Points;

	Total--;

	if (Total == 0)
	    Top=Bottom=Current=NULL;
    }

    return true;
}

