/*
 * Mouse handling routines
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The mouse interface to the program.
 *          The application must use this class to access
 *          the mouse (if any).
 *
 */


#include <stdio.h>
#include <stdlib.h>

#include "data.h"

#ifdef X_UNIX
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif

#ifdef HAVE_LIBGPM
#include <gpm.h>
#endif

#ifdef X_DOS
#include <dos.h>
union REGS Input,Output;
#endif

#include "mouse.h"

CMouse::CMouse()
{
    has_mouse = true;

#ifdef X_DOS
    Input.x.ax = 0x0000;
    int86(0x33,&Input,&Output);
    if (Output.x.ax == 0) { has_mouse = false; }
#elif defined(HAVE_LIBGPM) 

    Gpm_Connect tra;

    gpm_zerobased = 1;

    tra.eventMask = GPM_MOVE | GPM_DRAG | GPM_DOWN | GPM_UP;
    tra.defaultMask = 0;
    tra.minMod = 0;
    tra.maxMod = ~0;

    Gpm_Open(&tra,0);

    if (!gpm_flag || gpm_fd < 0) {
        has_mouse = false;
    }
#else
    has_mouse = false;
#endif

    Displayed=0;
}

CMouse::~CMouse()
{
    if (Displayed)
	Hide();

#ifdef HAVE_LIBGPM
    if (has_mouse)
        Gpm_Close();
#endif
}

int CMouse::Event()
{
    if (!has_mouse)
        return 0;

#ifdef HAVE_LIBGPM
    int ret;
    fd_set rf;
    struct timeval tv;

    FD_ZERO(&rf);
    FD_SET(gpm_fd,&rf);

    tv.tv_sec = 0;
    tv.tv_usec = 10;

    ret = select(gpm_fd+1,&rf,NULL,NULL,&tv);

    if (ret == -1) ret = 0;
    return ret;
#else
    return 1;
#endif
}

void CMouse::Display()
{
    if (!has_mouse)
        return;

#ifdef X_DOS
    Input.x.ax = 0x0001;
    int86(0x33,&Input,&Output);
#elif defined(HAVE_LIBGPM)
    GPM_DRAWPOINTER(&ev);
#endif
}

void CMouse::Hide()
{
    if (!has_mouse)
        return;
#ifdef X_DOS
    Input.x.ax = 0x0002;
    int86(0x33,&Input,&Output);
#endif
}

int CMouse::Get()
{
    int buttons;

    old_x = x;
    old_y = y;
    old_b1 = b1;
    old_b2 = b2;

    if (!has_mouse)
        return 0;

#ifdef X_DOS
    Input.x.ax = 0x0003;
    int86(0x33,&Input,&Output);
    
    buttons = Output.x.bx;
    x = Output.x.cx / 8;
    y = Output.x.dx / 8;

    b1 = (buttons & 1) ? 1 : 0;
    b2 = (buttons & 2) ? 1 : 0;

#elif defined(HAVE_LIBGPM)
    Gpm_GetEvent(&ev);
	
    x = ev.x;
    y = ev.y;
    b1 = (ev.buttons & GPM_B_LEFT) ? 1 : 0;
    b2 = (ev.buttons & GPM_B_RIGHT) ? 1 : 0;

    if (ev.type & GPM_UP) {
	if (b1) b1 = !b1;
	if (b2) b2 = !b2;
    }
#endif

    return EventType();
}

int CMouse::EventType()
{
    if (old_x != x || old_y != y) {
	return CMOUSE_MOVE;
    }

    if (!(old_b1) && (b1)) {
	b1 = 1;
	return CMOUSE_LEFT_DOWN;
    }
    else if ((old_b1) && !(b1)) {
	b1 = 0;
	return CMOUSE_LEFT_UP;
    }
    else if (!(old_b2) && (b2)) {
	b2 = 1;
	return CMOUSE_RIGHT_DOWN;
    }
    else if ((old_b2) && !(b2)) {
	b2 = 0;
	return CMOUSE_RIGHT_UP;
    }

    return 0;
}

void CMouse::Pointer(int p)
{

}
