/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The most basic classes used by the interface
 *          library.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "screen.h"
#include "xbase.h"


int THEME_LIGHT = COLOR_BLUE+H_INTENSITY;
int THEME_NORM = COLOR_BLUE;
int THEME_DARK = COLOR_BLACK+H_INTENSITY;


/**********************************************************************/


CPoint::CPoint(int xx,int yy)
{
    SetXY(xx,yy);
}
    
int CPoint::GetX()
{
    return x;
}

int CPoint::GetY()
{
    return y;
}
    
void CPoint::SetX(int xx)
{
    if (xx>=0 && xx<MAX_X)
	x = xx;
    else if (xx<0)
	x = 0;
    else
	x = MAX_X - 1;
}

void CPoint::SetY(int yy)
{
    if (yy>=0 && yy<=MAX_Y)
	y = yy;
    else if (yy<0)
	y = 0;
    else
	y = MAX_Y - 1;
}

void CPoint::SetXY(int xx,int yy)
{
    SetX(xx);
    SetY(yy);
}

void CPoint::Move(int dx,int dy)
{
    if (x+dx < MAX_X)
	SetX(x+dx);
    if (y+dy < MAX_Y)
	SetY(y+dy);
}



/**********************************************************************/

CRect::CRect()
{
    MinWidth = MIN_WIDTH;
    MinHeight = MIN_HEIGHT;

    c_x1 = 0;
    c_y1 = 0;
    c_x2 = MAX_X-1;
    c_y2 = MAX_Y-1;

    SetCoords(0,0,MinWidth,MinHeight);
}

CRect::CRect(int x1=0,int y1=0,int x2=MIN_WIDTH,int y2=MIN_HEIGHT)
{
    MinWidth = MIN_WIDTH;
    MinHeight = MIN_HEIGHT;
    
    c_x1 = 0;
    c_y1 = 0;
    c_x2 = MAX_X-1;
    c_y2 = MAX_Y-1;

    SetCoords(x1,y1,x2,y2);
}

void CRect::GetCoords(int &x1,int &y1,int &x2,int &y2)
{
    x1 = UpperLeft.GetX();
    y1 = UpperLeft.GetY();

    x2 = x1 + Width - 1;
    y2 = y1 + Height - 1;
}

void CRect::SetCoords(int x1,int y1,int x2,int y2)
{
    ClipRegion(x1,y1,x2,y2);
    UpperLeft.SetXY(x1,y1);

    Width = x2 - x1 + 1;
    Height = y2 - y1 + 1;
}

void CRect::SetCoordsNoClip(int x1,int y1,int x2,int y2)
{
    UpperLeft.SetXY(x1,y1);

    Width = x2 - x1 + 1;
    Height = y2 - y1 + 1;
}

int CRect::GetWidth()
{
    return Width;
}

int CRect::GetHeight()
{
    return Height;
}


void CRect::SetMinWidth(int n)
{
    MinWidth = n;
}

void CRect::SetMinHeight(int n)
{
    MinHeight = n;
}
 
void CRect::SetClipRegion(int x1,int y1,int x2,int y2)
{
    c_x1 = x1;
    c_y1 = y1;
    c_x2 = x2;
    c_y2 = y2;
}

void CRect::MoveD(int dx,int dy)
{
    int x1,y1,x2,y2;

    GetCoords(x1,y1,x2,y2);

    if (x2+dx <= c_x2) {
	x1+=dx;
	x2+=dx;
    }

    if (y2+dy <= c_y2) {
	y1+=dy;
	y2+=dy;
    }

    ClipRegion(x1,y1,x2,y2);
    SetCoords(x1,y1,x2,y2);
}

void CRect::ResizeD(int dx,int dy)
{
    int x1,y1,x2,y2;

    GetCoords(x1,y1,x2,y2);

    if (Width+dx >= MinWidth)
	x2 += dx;
    if (Height+dy >= MinHeight)
	y2 += dy;

    ClipRegion(x1,y1,x2,y2);
    SetCoords(x1,y1,x2,y2);
}
 
void CRect::Move(int x,int y)
{
    int x1,y1,x2,y2;

    x1 = x;
    y1 = y;
    x2 = x + Width - 1;
    y2 = y + Height - 1;

    ClipRegion(x1,y1,x2,y2);
    SetCoords(x1,y1,x2,y2);
}

void CRect::Resize(int x,int y)
{
    int x1,y1,x2,y2;

    GetCoords(x1,y1,x2,y2);

    x2 = x;
    y2 = y;

    ClipRegion(x1,y1,x2,y2);
    SetCoords(x1,y1,x2,y2);
}

void CRect::ClipRegion(int &x1,int &y1,int &x2,int &y2)
{
    int temp;

    if (x2 < x1)
	x2 = x1 + MIN_WIDTH - 1;
    if (y2 < y1)
	y2 = y1 + MIN_HEIGHT - 1;

    if (x1 < c_x1) {
	temp = c_x1 - x1;
	x1 += temp;
	x2 += temp;
    }
    if (x1 + MinWidth > c_x2) {
	temp = -(x1 + MinWidth - c_x2);
	x1 += temp;
	x2 += temp;
    }

    if (y1 < c_y1) {
	temp = c_y1 - y1;
	y1 += temp;
	y2 += temp;
    }
    if (y1 + MinHeight > c_y2) {
	temp = -(y1 + MinHeight - c_y2);
	y1 += temp;
	y2 += temp;
    }

    if (x2 > c_x2) {
	temp = -(x2 - c_x2);
	x2 += temp;
    }
    if (y2 > c_y2) {
	temp = -(y2 - c_y2);
	y2 += temp;
    }

}

int CRect::IsIn(CPoint p)
{
    int x1,y1,x2,y2;
    GetCoords(x1,y1,x2,y2);

    if (p.x >= x1 && p.y >= y1 && p.x <= x2 && p.y <= y2)
	return 1;
    else
	return 0;
}

int CRect::IsIn(int x,int y)
{
    int x1,y1,x2,y2;
    GetCoords(x1,y1,x2,y2);

    if (x >= x1 && y >= y1 && x <= x2 && y <= y2)
	return 1;
    else
	return 0;
}


int CRect::IsIn(CRect r)
{
    int x1,y1,x2,y2;
    int mx1,my1,mx2,my2;

    r.GetCoords(mx1,my1,mx2,my2);
    GetCoords(x1,y1,x2,y2);

    if (mx1>=x1 && my1>=y1 && mx2<=x2 && my2<=y2)
	return 1;
    else
	return 0;
}

int CRect::IsIn(int mx1,int my1,int mx2,int my2)
{
    int x1,y1,x2,y2;

    GetCoords(x1,y1,x2,y2);

    if (mx1>=x1 && my1>=y1 && mx2<=x2 && my2<=y2)
	return 1;
    else
	return 0;
}
