/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The button control of the interface library
 *
 */

#include <string.h>
#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "xinter.h"
#include "screen.h"
#include "xdraw.h"
#include "xcontrol.h"


#if defined(X_UNIX) || defined(X_DJGPP)
#undef KEY_ENTER
#define KEY_ENTER 10
#endif


/*
 * The user only needs to set the Property.Rect (only the upper left corner
 * counts. And Also the Title of the window. Its the text that appears on
 * the button.
 *
 */

CButton::CButton()
{
    Enabled = true;
    MouseOver = false;
}

bool CButton::OnCreate()
{
    return true;
}

bool CButton::OnPreCreate()
{
    int x1,y1,x2,y2;
    CRect r;

    if (!Property.Title)
	return false;

    Property.Rect.GetCoords(x1,y1,x2,y2);

    x2 = x1 + strlen(Property.Title) + 1;
    y2 = y1 + 1;

    Property.Rect.SetCoords(x1,y1,x2,y2);

    Property.Style = CWIN_TITLE;
    
    return true;
}

bool CButton::OnClose()
{
    return true;
}

bool CButton::OnPaint()
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    MyDraw.Rectangle(r,CBUTTON_LNORM);
    MyScreen.SetColor(CBUTTON_TNORM);
    MyScreen.PrintXY(x1,y1,Property.Title);

    return true;
}

bool CButton::OnGetFocus()
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    MyScreen.SetColor(CBUTTON_THIGH);
    MyScreen.PrintXY(x1,y1,Property.Title);


    return true;
}

bool CButton::OnLoseFocus()
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    MyScreen.SetColor(CBUTTON_TNORM);
    MyScreen.PrintXY(x1,y1,Property.Title);

    return true;
}

bool CButton::OnKeyDown(int Key,int Status)
{
    if (Property.Response && (Key == KEY_ENTER || Key == ' ')) {
	Property.Response->OnCommand(Property.Handle,RE_KEYDOWN,CPoint(0,0),
				     Key,Status);
    }

    return true;
}

bool CButton::OnLeftDown(CPoint p,int Status)
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    MyDraw.Rectangle(r,CBUTTON_LHIGH);
    MyScreen.SetColor(CBUTTON_THIGH);
    MyScreen.PrintXY(x1,y1,Property.Title);

    return true;
}

bool CButton::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CButton::OnLeftUp(CPoint p,int Status)
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    MyDraw.Rectangle(r,CBUTTON_LNORM);
    MyScreen.SetColor(CBUTTON_TNORM);
    MyScreen.PrintXY(x1,y1,Property.Title);


    if (Property.Response) {
	Property.Response->OnCommand(Property.Handle,RE_LEFTUP,p,0,Status);
    }

    return true;
}

bool CButton::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CButton::OnMove(CPoint p,int Status)
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    if (!r.IsIn(p) && MouseOver == true) {
	MyScreen.SetColor(CBUTTON_TNORM);
	MyScreen.PrintXY(x1,y1,Property.Title);
	MouseOver = false;
    }
    else if (r.IsIn(p) && MouseOver == false) {
	MyScreen.SetColor(CBUTTON_THIGH);
	MyScreen.PrintXY(x1,y1,Property.Title);
	MouseOver = true;
    }

    return true;
}

void CButton::Enable()
{
    Enabled = true;
}

void CButton::Disable()
{
    Enabled = false;
}
