/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The checkbox control of the interface library
 *
 */

#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "screen.h"
#include "xinter.h"

/*
 * For this class the user only needs to set the Property.Rect (only the x1
 * and y1 count) and call SetText(char*) to set the label of the box.
 * No other styles etc need to be set. It handles that itself.
 *
 */

CCheckBox::CCheckBox()
{
    State = false;
    Text = NULL;
}

CCheckBox::CCheckBox(const CCheckBox &obj)
{
    *this = obj;

    if (obj.Text) {
	Text = new char [strlen(obj.Text)+1];
	strcpy(Text,obj.Text);
    }
}

CCheckBox::~CCheckBox()
{
    if (Text)
	delete [] Text;
}

bool CCheckBox::OnCreate()
{
    return true;
}

bool CCheckBox::OnPreCreate()
{
    int x1,y1,x2,y2;
    int Len;

    Len = 3;
    if (Text)
	Len += strlen(Text) + 1;

    Property.Rect.GetCoords(x1,y1,x2,y2);
    x2 = x1 + Len - 1;
    y2 = y1;
    Property.Rect.SetCoords(x1,y1,x2,y2);

    Property.Style = 0;

    return true;
}

bool CCheckBox::OnClose()
{
    return true;
}

bool CCheckBox::OnPaint()
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetColor(CCHECKBOX_SELCOL);
    MyScreen.PutCharXY(x1,y1,'(');
    MyScreen.PutCharXY(x1+2,y1,')');
    
    MyScreen.SetColor(State ? CCHECKBOX_SELCOL : CCHECKBOX_UNSELCOL);
    MyScreen.PutCharXY(x1+1,y1,'x');
    
    if (Text) {
	MyScreen.SetColor(CCHECKBOX_TEXTCOL1);
	MyScreen.PutCharXY(x1+4,y1,*Text);
	MyScreen.SetColor(CCHECKBOX_TEXTCOL2);
	MyScreen.PrintXY(x1+5,y1,Text+1);
    }

    return true;
}

bool CCheckBox::OnGetFocus()
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetColor(CCHECKBOX_SELCOL);
    MyScreen.PutCharXY(x1,y1,'(');
    MyScreen.PutCharXY(x1+2,y1,')');

    return true;
}

bool CCheckBox::OnLoseFocus()
{
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetColor(CCHECKBOX_UNSELCOL);
    MyScreen.PutCharXY(x1,y1,'(');
    MyScreen.PutCharXY(x1+2,y1,')');

    return true;
}

bool CCheckBox::OnKeyDown(int Key,int Status)
{
    State = !State;
    OnPaint();

    return true;
}

bool CCheckBox::OnLeftDown(CPoint p,int Status)
{
    State = !State;
    OnPaint();

    return true;
}

bool CCheckBox::OnRightDown(CPoint p,int Status)
{
    State = !State;
    OnPaint();

    return true;
}

bool CCheckBox::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CCheckBox::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CCheckBox::OnMove(CPoint p,int Status)
{
    return true;
}

bool CCheckBox::GetState()
{
    return State;
}

void CCheckBox::SetState(int s)
{
    if (s)
	State=true;
    else
	State=false;

}

void CCheckBox::SetText(char *str)
{
    int Len;

    if (Text)
	delete [] Text;

    Len = strlen(str);

    Text = new char [Len+1];
    strcpy(Text,str);

}
