/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - Some useful drawing functions used by the interface
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "screen.h"
#include "xbase.h"
#include "xdraw.h"

CDraw::CDraw()
{

}

CDraw::~CDraw()
{

}

void CDraw::DrawKewlBox(CRect r)
{
    int i;
    int Say; 
    int Color;
    int MinCol,MinRow,MaxCol,MaxRow;

    srandom(21);

    r.GetCoords(MinCol,MinRow,MaxCol,MaxRow);

    MyScreen.SetCursor(MinCol,MinRow);
    MyScreen.SetColor(BOXLIGHT);

    MyScreen.PutChar(ACS_ULCORNER);

    for (i=0;i<MaxCol-MinCol;i++) {
	if (i>=0 && i<=3) Color=BOXLIGHT;
	else if (i>=4 && i<=7) Color=BOXNORM;
	else Color=BOXDARK;
	if ((random()%15)<4) Say='-'; else Say=ACS_HLINE;
	if (i+MinCol>MaxCol-4 && i+MinCol<MaxCol) Say=' ';
	if (i==MaxCol-MinCol-1) Say=249;
	
	MyScreen.SetColor(Color);
	MyScreen.PutChar(Say);
    }

    for (i=1;i<MaxRow-MinRow;i++) {
	if (i==2) Say='|'; else Say=ACS_VLINE;
	if (i+MinRow==MaxRow-2) Say='.'; else if (i+MinRow>MaxRow-2) Say=' ';
	if (i==1) Color=BOXLIGHT;
	else if (i>=2 && i<=3) Color=BOXNORM;
	else Color=BOXDARK;

	MyScreen.SetCursor(MinCol,i+MinRow); 
	MyScreen.SetColor(Color); 
	MyScreen.PutChar(Say);

	if (i==1 || i==2) Say=' '; 
	else if ((random()%15)<3) Say='|'; 
	else Say=ACS_VLINE;

	if (i==3) Say='.';
	if (i+MinRow>=MaxRow-1 && i+MinRow<MaxRow) Color=BOXLIGHT;
	else if (i+MinRow>=MaxRow-3 && i+MinRow<MaxRow-1) Color=BOXNORM;
	else Color=BOXDARK;
	
	MyScreen.SetCursor(MaxCol,i+MinRow); 
	MyScreen.SetColor(Color); 
	MyScreen.PutChar(Say);
    }

    MyScreen.SetCursor(MinCol+1,MaxRow);
    for (i=0;i<MaxCol-MinCol-1;i++) {
	if (i+MinCol>=MaxCol-3 && i+MinCol<=MaxCol) Color=BOXLIGHT;
	else if (i+MinCol>=MaxCol-8 && i+MinCol<=MaxCol-4) Color=BOXNORM;
	else Color=BOXDARK;
	if ((random()%15)<4) Say='-'; else Say=ACS_HLINE;
	if (i==0) Say=250;
	
	MyScreen.SetColor(Color);
	MyScreen.PutChar(Say);
    }
    
    MyScreen.SetColor(BOXLIGHT); 
    MyScreen.PutChar(ACS_LRCORNER);

}

void CDraw::KewlVLine(int MinRow,int MaxRow,int MinCol)
{
    int i;
    int Say; 
    int Color;


    for (i=0;i<MaxRow-MinRow;i++) {
	if (i==2) Say='|'; else Say=ACS_VLINE;
	if (i+MinRow==MaxRow-2) Say='.'; else if (i+MinRow>MaxRow-2) Say=' ';
	if (i<=1) Color=BOXLIGHT;
	else if (i>=2 && i<=3) Color=BOXNORM;
	else Color=BOXDARK;

	MyScreen.SetCursor(MinCol,i+MinRow); 
	MyScreen.SetColor(Color); 
	MyScreen.PutChar(Say);
    }
}

void CDraw::KewlHLine(int MinCol,int MaxCol,int MinRow)
{
    int i;
    int Say; 
    int Color;


    MyScreen.SetCursor(MinCol,MinRow);

    for (i=0;i<MaxCol-MinCol;i++) {
	if (i>=0 && i<=3) Color=BOXLIGHT;
	else if (i>=4 && i<=7) Color=BOXNORM;
	else Color=BOXDARK;
	if ((random()%15)<4) Say='-'; else Say=ACS_HLINE;
	if (i+MinCol>MaxCol-4 && i+MinCol<MaxCol) Say=' ';
	if (i==MaxCol-MinCol-1) Say=249;
	
	MyScreen.SetColor(Color);
	MyScreen.PutChar(Say);
    }
}


void CDraw::Rectangle(CRect r,int color)
{
    int x1,y1,x2,y2;
    int i;

    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetColor(color);
    MyScreen.SetCursor(x1,y1);

    MyScreen.PutChar(ACS_ULCORNER);
    for (i=x1+1;i<x2;i++) MyScreen.PutChar(ACS_HLINE);
    MyScreen.PutChar(ACS_URCORNER);

    for (i=y1+1;i<y2;i++) {
	MyScreen.SetCursor(x1,i);
	MyScreen.PutChar(ACS_VLINE);
	
	MyScreen.SetCursor(x2,i);
	MyScreen.PutChar(ACS_VLINE);
    }

    MyScreen.SetCursor(x1,y2);
    MyScreen.PutChar(ACS_LLCORNER);
    for (i=x1+1;i<x2;i++) MyScreen.PutChar(ACS_HLINE);
    MyScreen.PutChar(ACS_LRCORNER);
    
}

void CDraw::RectangleUnactive(CRect r)
{
    Rectangle(r,BOXUNACTIVE);
}

void CDraw::HLine(CPoint a,CPoint b,int c)
{
    int i;

    if (a.y != b.y)
	return;

    MyScreen.SetColor(c);
    MyScreen.SetCursor(a.x,a.y);
    for (i=a.x;i<=b.x;i++)
	MyScreen.PutChar(ACS_HLINE);
}

void CDraw::VLine(CPoint a,CPoint b,int c)
{
    int i;

    if (a.x != b.x)
	return;

    MyScreen.SetColor(c);
    for (i=a.y;i<=b.y;i++) {
	MyScreen.SetCursor(a.x,i);
	MyScreen.PutChar(ACS_VLINE);
    }
}

void CDraw::VLine(int y1,int y2,int x,int c)
{
    int i;

    MyScreen.SetColor(c);
    for (i=y1;i<=y2;i++) {
	MyScreen.SetCursor(x,i);
	MyScreen.PutChar(ACS_VLINE);
    }
}

void CDraw::HLine(int x1,int x2,int y,int c)
{
    int i;

    MyScreen.SetColor(c);
    MyScreen.SetCursor(x1,y);
    for (i=x1;i<=x2;i++)
	MyScreen.PutChar(ACS_HLINE);
}

void CDraw::TextCenter(CPoint a,CPoint b,char *str,int color)
{
    int i,len;
    char *Points;
    
    MyScreen.SetColor(color);

    len = strlen(str);

    if (b.x - a.x + 1 <= len) {
	Points = str;
	for (i=a.x + 1;i<b.x;i++) {
	    MyScreen.SetCursor(i,a.y);
	    MyScreen.PutChar(*Points);
	    Points++;
	}
    }
    else {
	i = (b.x - a.x + 1) / 2;
	i += a.x;
	i -= len / 2;

	MyScreen.SetCursor(i,a.y);
	MyScreen.Print("%s",str);
    }
}
