/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The editbox control of the interface
 *
 */


#include <stdlib.h>
#include <string.h>


#include "data.h"
#include "xinter.h"
#include "xcontrol.h"
#include "screen.h"

#if defined(X_UNIX) || defined(X_DJGPP)
#undef KEY_ENTER
#define KEY_ENTER 10
#endif

#ifdef X_DJGPP
#undef KEY_BACKSPACE
#define KEY_BACKSPACE 8
#endif

/*
 * The height of this box must be one. The user only needs to set the
 * property.rect for this class. The rest if handled by the editbox
 * itself.
 *
 */

CEditBox::CEditBox()
{
    Buffer = NULL;
    CharSet = NULL;

    Cursor = 0;
    Scroll = 0;
    Length = 0;

    Property.Style = 0;

    IsCreated = false;
}

CEditBox::CEditBox(const CEditBox &obj)
{
    *this = obj;

    if (obj.Buffer) {
	Buffer = new char [strlen(obj.Buffer)+1];
	strcpy(Buffer,obj.Buffer);
    }
    else {
	Buffer = NULL;
    }

    if (obj.CharSet) {
	CharSet = new char [strlen(obj.CharSet)+1];
	strcpy(CharSet,obj.CharSet);
    }
    else {
	CharSet = NULL;
    }
}

CEditBox::~CEditBox()
{
    if (Buffer)
	delete [] Buffer;

    if (CharSet)
	delete [] CharSet;
}

bool CEditBox::OnCreate()
{
    int i;

    if (!CharSet) {
	CharSet = new char [strlen(CEDIT_ALPHANUM)+1];
	strcpy(CharSet,CEDIT_ALPHANUM);
    }
    
    if (!Buffer) {
	if (Length <= 0)
	    Length = 20;

	Buffer = new char [Length+1];
	Buffer[Length] = '\0';

	for (i=0;i<Length;i++)
	    Buffer[i] = ' ';
    }
    else if (Length < strlen(Buffer)) {
	Length = strlen(Buffer);
    }

    IsCreated = true;
    return true;
}

bool CEditBox::OnPreCreate()
{
    return true;
}

bool CEditBox::OnClose()
{
    return true;
}

bool CEditBox::OnPaint()
{
    CRect r;
    int x1,y1,x2,y2;
    int i;
    char *Points;

    if (!CharSet || !Buffer)
	return false;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.Clear(x1,y1,x2,y2);

    MyScreen.SetColor(CEDIT_ENDCOL1);
    MyScreen.PutCharXY(x1,y1,'(');
    MyScreen.PutCharXY(x2,y2,')');


    MyScreen.SetColor(CEDIT_TEXTCOL);
    Points = Buffer + Scroll;

    for (i=x1+1;i<x2;i++) {
	if (!(*Points))
	    break;

	MyScreen.PutCharXY(i,y1,*Points);
	Points++;

    }

    MyScreen.SetCursor(x1+Cursor-Scroll+1,y1);

    return true;
}

bool CEditBox::OnGetFocus()
{
    CRect r;
    int x1,y1,x2,y2;
    int i;
    char *Points;

    if (!CharSet || !Buffer)
	return false;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetColor(CEDIT_ENDCOL1);
    MyScreen.PutCharXY(x1,y1,'(');
    MyScreen.PutCharXY(x2,y2,')');

    MyScreen.SetCursor(x1+Cursor-Scroll+1,y1);

    return true;
}

bool CEditBox::OnLoseFocus()
{
    CRect r;
    int x1,y1,x2,y2;
    int i;
    char *Points;

    if (!CharSet || !Buffer)
	return false;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetColor(CEDIT_ENDCOL2);
    MyScreen.PutCharXY(x1,y1,'(');
    MyScreen.PutCharXY(x2,y2,')');

    return true;
}

bool CEditBox::OnKeyDown(int Key,int Status)
{
    CRect r;
    int x1,y1,x2,y2;
    int Width;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    Width = r.GetWidth();


    if (!CharSet || !Buffer) {
	return false;
    }

    switch (Key) {
	
    case KEY_ENTER:
	break;

    case KEY_LEFT : 
	if (Cursor==0) 
	    return false;
	else if (Cursor==Scroll) { 
	    Scroll--; 
	    Cursor--; 
	}
	else Cursor--;
	break;

    case KEY_RIGHT :
	if (Cursor == Length-1)
	    return false;
	else if (Cursor==Scroll+Width-3) {
	    Cursor++;
	    Scroll++;
	}
	else Cursor++;
	break;

    case KEY_BACKSPACE :
	if (Cursor == 0)
	    return false;
	else if (Cursor==Scroll) { 
	    Scroll--; 
	    Cursor--; 
	}
	else Cursor--;

	Delch(Cursor);
	break;

    case KEY_DC :
	Delch(Cursor);
	break;
	    
    case KEY_HOME :
	Cursor = 0;
	Scroll = 0;
	break;

    case KEY_END :
	Cursor = Length-1;
	Scroll = Length-Width+2;

        while (Buffer[Cursor]==' ' && Cursor) {
            if (Buffer[Cursor-1] != ' ')
                break;
            Cursor--;
            if (Scroll>0) Scroll--;
        }

	break;

    default :
	if (!strchr(CharSet,Key))
	    return false;
	
	Insch(Cursor);
	Buffer[Cursor] = Key;

	if (Cursor == Length-1)
	    break;
	else if (Cursor==Scroll+Width-3) {
	    Cursor++;
	    Scroll++;
	}
	else Cursor++;

	break;

    }

    OnPaint();

    if (Property.Response) {
	Property.Response->OnCommand(Property.Handle,
				     RE_KEYDOWN,CPoint(0,0),Key,Status);
    }

    return true;
}

bool CEditBox::OnLeftDown(CPoint p,int Status)
{
    CRect r;
    int x1,y1,x2,y2;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyScreen.SetCursor(x1+Cursor-Scroll+1,y1);

    return true;
}

bool CEditBox::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CEditBox::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CEditBox::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CEditBox::OnMove(CPoint p,int Status)
{
    return true;
}

bool CEditBox::GetText(char *str)
{
    int i;

    strcpy(str,Buffer);

    for (i=Length-1;i>=0;i--) {
	if (str[i] == ' ')
	    str[i] = '\0';
	else
	    break;
    }

    return true;
}

bool CEditBox::SetText(char *str)
{
    if (Buffer)
	delete [] Buffer;

    if (Length < strlen(str))
	Length = strlen(str);

    Buffer = new char [Length+1];
    memset(Buffer,' ',Length);

    Buffer[Length] = '\0';

    for (int i=0;i<Length;i++) {
	if (!str[i]) break;
	Buffer[i] = str[i];
    }

    return true;
}
    
bool CEditBox::SetCharSet(char *set)
{
    if (IsCreated)
	return false;

    if (CharSet)
	delete [] CharSet;

    CharSet = new char [strlen(set)+1];
    strcpy(CharSet,set);

    return true;
}

bool CEditBox::SetLength(int n)
{
    if (IsCreated)
	return false;

    Length = n;
}

int CEditBox::GetLength()
{
    return Length;
}

void CEditBox::Delch(int n)
{
    int i;

    for (i=n+1;i<Length;i++) {
	Buffer[i-1] = Buffer[i];
    }

    Buffer[Length-1] = ' ';
}

void CEditBox::Insch(int n)
{
    int i;

    for (i=Length-1;i>n;i--) {
	Buffer[i] = Buffer[i-1];
    }

    Buffer[n] = ' ';
}
