/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The listbox control of the interface
 *
 */

#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "xinter.h"
#include "screen.h"
#include "xdraw.h"
#include "xcontrol.h"


#if defined(X_UNIX) || defined(X_DJGPP)
#undef KEY_ENTER
#define KEY_ENTER 10
#endif



/*
 * For a listbox, set Property.Rect. Add all the items using 
 * listbox::additem(char*) and there must atleast be one item in the
 * list. No other styles need to be set.
 * The scrollable option and the additems must be called before the
 * window is created. It also returns response on leftdown in the list
 * area or pressing enter.
 * Add a '!' for a hline
 *
 * Valid styles are...
 * CWIN_BORDER CWIN_TITLE
 * Invalid styles are..
 * All others. Will cause errors setting any other styles.
 * 
 */

CListBox::CListBox() : Items(sizeof(char*))
{
    Select = 0;
    FollowMode = true;

    Scrollable = RemoveMode = ReorderMode = false;
    Scroll = 0;

    ScrollHandle = -1;
}

CListBox::~CListBox()
{

}

bool CListBox::OnCreate()
{
    CWindow::OnCreate();
}

bool CListBox::OnPreCreate()
{
    int w,h;
    int x1,y1,x2,y2;

    w = Property.Rect.GetWidth();
    h = Property.Rect.GetHeight();

    Property.Rect.GetCoords(x1,y1,x2,y2);

    if (Scrollable) {
	CScrollBar *SBar = new CScrollBar;
	SBar->Property.Rect.SetMinWidth(1);
	SBar->Property.Rect.SetMinHeight(1);
	SBar->Property.SetCoords(w-2,1,w-2,h-2);
	SBar->SetRange(0,Items.GetTotal()-1,0);

	SBar->Property.Response = this;

	ScrollHandle = AddControl(SBar);
    }
    
    CWindow::OnPreCreate();

    return true;
}

bool CListBox::OnClose()
{
    int i;
    char *Points;

    for (i=0;i<Items.GetTotal();i++) {
	Points = *(char **)Items.GetItem(i);

	delete [] Points;
    }

    Items.DeleteAll();

    CWindow::OnClose();

    return true;
}

bool CListBox::OnPaint()
{
    int i,j;
    int Width,Height;
    char *p;
    int x1,y1,x2,y2;
    CRect r;

    if (Scrollable) {
        int w = Property.Rect.GetWidth();
        int h = Property.Rect.GetHeight();
	CWindow *SBar = GetControl(ScrollHandle);
	SBar->Property.SetCoords(w-2,1,w-2,h-2);
    }

    CWindow::OnPaint();

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    Width = Property.Rect.GetWidth() - 2;
    if (Scrollable)
	Width--;

    Height = Property.Rect.GetHeight() - 2;


    for (i=0;i<Height;i++) {
	if (i+Scroll >= Items.GetTotal())
	    break;

	MyScreen.SetCursor(x1+2,y1+1+i);

	if (i+Scroll == Select)
	    MyScreen.SetColor(CLISTBOX_THIGHCOL);
	else
	    MyScreen.SetColor(CLISTBOX_TNORMCOL);


	p = *(char **) Items.GetItem(i+Scroll);

	if (*p == '!') {
	    MyDraw.KewlHLine(x1+2,x1+Width-2,y1+1+i);
	    continue;
	}

	for (j=0;j<Width - 2;j++) {
	    if (!(*p) || !p) break;
	    MyScreen.PutChar(*p);
	    p++;
	}
    }

    return true;
}

bool CListBox::OnGetFocus()
{
    CWindow::OnGetFocus();

    return true;
}

bool CListBox::OnLoseFocus()
{
    CWindow::OnLoseFocus();

    return true;
}

bool CListBox::OnKeyDown(int Key,int Status)
{
    int TotalItems;
    int Total;

    CWindow::OnKeyDown(Key,Status);

    if (CurrentControl != -1)
	return true;

    TotalItems = Items.GetTotal();
    Total = Property.Rect.GetHeight() - 2;


    if (Key == KEY_DOWN) {
	if (Select < TotalItems - 1) {
	    Select++;

	    if (Select == Scroll+Total)
		Scroll++;
	}
    }
    else if (Key == KEY_UP) {
	if (Select > 0) {
	    Select--;

	    if (Select+1 == Scroll)
		Scroll--;
	}
    }
    else if (Key == KEY_NPAGE) {
        if (Select+Total-1 <= TotalItems - 1)
            Select = Select+Total-1;
        else
            Select = TotalItems - 1;

        Scroll = (Scroll+Total-1 < TotalItems) ? Scroll+Total-1 : Select;
        if (TotalItems-Total > 0 && Scroll > TotalItems-Total)
            Scroll = TotalItems-Total;
    }
    else if (Key == KEY_PPAGE) {
        if (Select-Total+1 >= 0)
            Select = Select-Total+1;
        else
            Select = 0;

        Scroll = (Scroll-Total+1 >= 0) ? Scroll-Total+1 : 0;
    }
    else if (Key == CLISTBOX_DELKEY && RemoveMode && TotalItems) {
        char *str = *(char**)Items.GetItem(Select);

        if (str) delete [] str;

        if (Items.Delete(Select))
            TotalItems--;

        if (TotalItems) {
            if (Select >= TotalItems)
                Select = Items.GetTotal() - 1;
            if (Scroll >= TotalItems)
                Scroll = Items.GetTotal() - 1;
        }
        
        if (IsCreated && Scrollable) {
            CScrollBar *Points = (CScrollBar *)GetControl(ScrollHandle);
            Points->SetRange(0,Items.GetTotal()-1,Select);
        }
    }
    else if (Key == CLISTBOX_REORDER_BW && ReorderMode && TotalItems) {
	if (Select > 0) {
            char *str = *(char**)Items.GetItem(Select);
            Items.Delete(Select);
            Items.InsertItem(Select-1, &str);

	    Select--;
	    if (Select+1 == Scroll)
		Scroll--;
	}
    }
    else if (Key == CLISTBOX_REORDER_FW && ReorderMode && TotalItems) {
	if (Select < TotalItems - 1) {
            char *str = *(char**)Items.GetItem(Select);
            Items.Delete(Select);
            Items.InsertItem(Select+1, &str);

	    Select++;
	    if (Select == Scroll+Total)
		Scroll++;
	}
    }

    if (Scrollable) {
	CScrollBar *Points = (CScrollBar*) GetControl(ScrollHandle);
	
	Points->SetCurrent(Select);
    }

    OnPaint();

    if (Property.Response) {
	if (Key == KEY_UP || Key == KEY_DOWN || Key == KEY_ENTER ||
            Key == KEY_NPAGE || Key == KEY_PPAGE)
	    Property.Response->OnCommand(Property.Handle,RE_KEYDOWN,
                                         CPoint(0,0),Key,Status);
    }

    return true;
}

bool CListBox::OnLeftDown(CPoint p,int Status)
{
    int x1,y1,x2,y2;
    CRect r;
    int OSel;

    CWindow::OnLeftDown(p,Status);

    if (CurrentControl != -1)
	return true;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    x1++; y1++; x2--; y2--;
    
    if (Scrollable)
	x2--;

    r.SetMinWidth(1);
    r.SetMinHeight(1);
    r.SetCoords(x1,y1,x2,y2);

    if (r.IsIn(p)) {
	OSel = Select;

	if (FollowMode == false)
	    Select = Scroll + p.y - y1;

	if (Select >= Items.GetTotal()) 
	    Select = OSel;

	if (Scrollable) {
	    CScrollBar *Points = (CScrollBar *)GetControl(ScrollHandle);
	    Points->SetCurrent(Select);
	}

	if (OSel != Select)
	    OnPaint();

	if (Property.Response)
	    Property.Response->OnCommand(Property.Handle,RE_LEFTDOWN,
					 p,0,Status);
    }

    return true;
}

bool CListBox::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CListBox::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CListBox::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CListBox::OnMove(CPoint p,int Status)
{
    int x1,y1,x2,y2;
    CRect r;

    if (FollowMode == false)
	return true;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    x1++; y1++; x2--; y2--;
    
    if (Scrollable)
	x2--;

    r.SetMinWidth(1);
    r.SetMinHeight(1);
    r.SetCoords(x1,y1,x2,y2);

    if (r.IsIn(p)) {
	if (Select == p.y - y1 + Scroll)
	    return true;
	else
	    Select = p.y - y1 + Scroll;

	if (Select >= Items.GetTotal())
	    Select = Items.GetTotal() - 1;

	if (Scrollable) {
	    CScrollBar *Points = (CScrollBar *)GetControl(ScrollHandle);
	    
	    Points->SetCurrent(Select);
	}

	OnPaint();
    }

    return true;
}

bool CListBox::OnCommand(int Handle,long int Event,CPoint p,int Key,
		       int Status)
{
    int Total;
    int x1,y1,x2,y2;
    CRect r;

    if (Handle != ScrollHandle)
	return false;

    Total = Property.Rect.GetHeight() - 2;

    CScrollBar *Points = (CScrollBar *)GetControl(CurrentControl);
    Select = Points->GetCurrent();

    r = MapToScreen(Points->Property.Rect,Points);
    r.GetCoords(x1,y1,x2,y2);


    if ((Event == RE_KEYDOWN && Key == KEY_UP) || 
	(Event == RE_LEFTDOWN && p.y == y1)) {

	if (Select+1 == Scroll)
	    Scroll--;
    }
    else if ((Event == RE_KEYDOWN && Key == KEY_DOWN) || 
	     (Event == RE_LEFTDOWN && p.y == y2)) {
	if (Select == Scroll+Total)
	    Scroll++;
    }
	
    OnPaint();
}



void CListBox::SetScrollable(bool val)
{
    if (!IsCreated)
	Scrollable = val;
}

void CListBox::SetFollow(bool val)
{
    FollowMode = val;
}

void CListBox::SetRemovable(bool val)
{
    RemoveMode = val;
}

void CListBox::SetReorder(bool val)
{
    ReorderMode = val;
}

bool CListBox::AddItem(char *s)
{
    char *p;

    p = new char [strlen(s)+1];
    strcpy(p,s);

    Items.AddTop(&p);

    if (IsCreated && Scrollable) {
	CScrollBar *Points = (CScrollBar *)GetControl(ScrollHandle);
	Points->SetRange(0,Items.GetTotal()-1,0);
    }

    return true;
}

bool CListBox::DeleteAll()
{
    int i;
    char *Points;

    for (i=0;i<Items.GetTotal();i++) {
	Points = *(char **)Items.GetItem(i);
	delete [] Points;
    }

    Items.DeleteAll();

    Select = 0;
    Scroll = 0;

    return true;
}

void CListBox::SetSelect(int s)
{
    int Total = Property.Rect.GetHeight() - 2;

    if (s >= 0 && s < Items.GetTotal()) {
	Select = s;

	if (Select < Scroll || Select >= Scroll+Total)  
	    Scroll = Select;

	if (Scrollable) {
	    CScrollBar *Points = (CScrollBar*) GetControl(ScrollHandle);
	    Points->SetCurrent(Select);
	}

    }
}

int CListBox::GetSelect()
{
    return Select;
}

char *CListBox::GetItem()
{
    char *Points;

    Points = *(char **)Items.GetItem(Select);
    return Points;
}

char *CListBox::GetItem(int n)
{
    if (n<0 || n>=Items.GetTotal())
        return NULL;

    char *retval = *(char**)Items.GetItem(n);
    return retval;
}

CContainer *CListBox::GetItemContainer()
{
    return &Items;
}

int CListBox::GetTotal()
{
    return Items.GetTotal();
}
