/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The menu control of the interface library
 *
 */

#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "xinter.h"
#include "screen.h"
#include "xdraw.h"
#include "xcontrol.h"


#if defined(X_UNIX) || defined(X_DJGPP)
#define KEY_ESC 27
#undef KEY_ENTER
#define KEY_ENTER 10
#endif

/*
 * Make a listbox and add it to the menu after adding the menu as a control
 * User needs to set property.rect
 * and it should be just two line. Returns response to itself. No
 * styles count.
 * Add menu first to the window and then add the items to the menu. And 
 * then add the rest of the items to the window.
 *
 * In window..add menu in oncreate (but 1st call cwindow::oncreate) and 
 * then add the rest of the controls.
 * and in cmenu derived class add lists in onpre and then call cmenu::onpre
 * and in oncommand...call cmenu::oncommand.
 *
 */

CMenuNode::CMenuNode(char *str,int h)
{
    Text = new char [strlen(str)+1];
    strcpy(Text,str);

    Handle = h;
}

CMenuNode::~CMenuNode()
{
    if (Text)
	delete [] Text;

    Text = NULL;
    Handle = -1;
}

/**********************************************************************/

CMenu::CMenu() : List(sizeof(CMenuNode*))
{
    Select = -1;
}

CMenu::~CMenu()
{

}

bool CMenu::OnCreate()
{
    CWindow::OnCreate();
    return true;
}

bool CMenu::OnPreCreate()
{
    int x1,y1,x2,y2;
    int i;
    CMenuNode *ptr;

    Property.Rect.GetCoords(x1,y1,x2,y2);
    y2 = y1+1;
    x2 = x1 + 2;

    for (i=0;i<List.GetTotal();i++) {
	ptr = *(CMenuNode **) List.GetItem(i);

	x2 += strlen(ptr->Text) + 2;
    }

    Property.Rect.SetCoords(x1,y1,x2,y2);

    return true;
}

bool CMenu::OnClose()
{
    int i;
    CMenuNode *ptr;

    for (i=0;i<List.GetTotal();i++) {
	ptr = *(CMenuNode **) List.GetItem(i);
	delete ptr;
    }

    List.DeleteAll();
    return true;
}

bool CMenu::OnPaint()
{
    int i;
    CMenuNode *ptr;
    int x1,y1,x2,y2;
    CRect r;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    MyDraw.Rectangle(CRect(x1,y1,x2,y2),CMENU_BCOL);
    MyScreen.SetCursor(x1+2,y2);

    for (i=0;i<List.GetTotal();i++) {
	ptr = *(CMenuNode **) List.GetItem(i);

	if (Select == ptr->Handle)
	    MyScreen.SetColor(CMENU_SELCOL);
	else
	    MyScreen.SetColor(CMENU_UNSELCOL);

	MyScreen.Print("%s",ptr->Text);

	MyScreen.SetColor(CMENU_BCOL);
	MyScreen.PutChar(ACS_HLINE);
	MyScreen.PutChar(ACS_HLINE);
    }

    return true;
}

bool CMenu::OnGetFocus()
{
    CWindow::OnGetFocus();

    return true;
}

bool CMenu::OnLoseFocus()
{
    CWindow::OnLoseFocus();

    return true;
}

bool CMenu::OnKeyDown(int Key,int Status)
{
    int i;
    CListBox *box;
    int Osel;
    CMenuNode *ptr;
    int older=-1;

    Osel = Select;

    switch (Key) {
    case KEY_LEFT:
	for (i=0;i<List.GetTotal();i++) {
	    if (i != 0) older = ptr->Handle;
	    ptr = *(CMenuNode **) List.GetItem(i);
	    if (ptr->Handle == Select && older != -1) {
		Select = older;
		break;
	    }
	}
	break;

    case KEY_RIGHT: 
	for (i=List.GetTotal()-1;i>=0;i--) {
	    if (i != List.GetTotal()-1) older = ptr->Handle;
	    ptr = *(CMenuNode **) List.GetItem(i);
	    if (ptr->Handle == Select && older != -1) {
		Select = older;
		break;
	    }
	}
	break;

    case KEY_ESC:
	Select = -1;
	break;
/*
    case KEY_ENTER:
    case KEY_DOWN:
    case KEY_UP:
	if (Select != -1) {
	    box = (CListBox *)
		((CWindow *)Property.Parent)->GetControl(Select);
	    box->OnKeyDown(Key,Status);
	}
	break;
*/
    }

    if (Osel != Select && Osel != -1) {
	box = (CListBox *) 
	    ((CWindow *)Property.Parent)->GetControl(Osel);
	box->Property.Visible = false;

	if (Select != -1) {
	    box = (CListBox *) 
		((CWindow *)Property.Parent)->GetControl(Select);

	    box->Property.Visible = true;
	}

	Property.Parent->OnPaint();
    }


    return true;
}

bool CMenu::OnLeftDown(CPoint p,int Status)
{
    int i,Osel;
    int x1,y1,x2,y2,dx;
    CRect r;
    CMenuNode *ptr;
    CListBox *box;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    dx = x1+2;
    Osel = Select;

    if (p.y != y2) return true;

    for (int i=0;i<List.GetTotal();i++) {
	ptr = *(CMenuNode **) List.GetItem(i);

	if (p.x>=dx && p.x<dx+strlen(ptr->Text)) {
	    Select = ptr->Handle;
	    break;
	}
	
	dx+=strlen(ptr->Text)+2;
    }

    if (Osel != Select && Osel != -1) {
	box = (CListBox *) 
	    ((CWindow *)Property.Parent)->GetControl(Osel);

	box->Property.Visible = false;
	Property.Parent->OnPaint();
    }
    if (Select != -1) {
	box = (CListBox *) 
	    ((CWindow *)Property.Parent)->GetControl(Select);

	box->Property.Visible = true;
	Property.Parent->OnPaint();
    }

    return true;
}

bool CMenu::OnRightDown(CPoint p,int Status)
{
    CWindow::OnRightDown(p,Status);
    return true;
}

bool CMenu::OnLeftUp(CPoint p,int Status)
{
    CWindow::OnLeftUp(p,Status);
    return true;
}

bool CMenu::OnRightUp(CPoint p,int Status)
{
    CWindow::OnRightUp(p,Status);
    return true;
}

bool CMenu::OnMove(CPoint p,int Status)
{
    return true;
}

bool CMenu::OnCommand(int Handle,long int Event,CPoint p,int Key,
		       int Status)
{
    CWindow *win;

    if (Event == RE_LEFTDOWN || (Event == RE_KEYDOWN && Key == KEY_ENTER)) {
	Select = -1;

	win = (CWindow *)((CWindow *)Property.Parent)->GetControl(Handle);
	win->Property.Visible = false;

	Property.Parent->OnLoseFocus();
	Property.Parent->OnGetFocus();
	Property.Parent->OnPaint();
    }

    return true;
}


int CMenu::AddItem(char *str,CListBox *box)
{
    int h;
    CMenuNode *ptr;
    CWindow *win;
    int x1,y1,x2,y2;
    int dx;

    if (IsCreated || !Property.Parent)
	return -1;

    Property.Rect.GetCoords(x1,y1,x2,y2);
    dx = x1+2;

    for (int i=0;i<List.GetTotal();i++) {
	ptr = *(CMenuNode **) List.GetItem(i);
	dx += strlen(ptr->Text) + 2;
    }

    box->Property.Rect.Move(dx,y1+2);
    box->Property.Response = this;
    box->Property.Visible = false;
    h = ((CWindow *)Property.Parent)->AddControl(box);

    ptr = new CMenuNode(str,h);
    List.AddTop(&ptr);

    return h;
}
