/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The progress control of the interface library
 *
 */

#include <stdlib.h>

#include "data.h"
#include "xinter.h"
#include "xdraw.h"


/*
 * For a progress bar, the user only needs to set the Property.Rect.
 * No other styles or properties need to be set. The progress bar
 * returns no response. It only displays the progress.
 *
 */


CProgress::CProgress()
{
    Min = 0;
    Max = 100;
    Current = 0;
}

CProgress::~CProgress()
{

}

bool CProgress::OnCreate()
{
    IsCreated = true;

    return true;
}

bool CProgress::OnPreCreate()
{
    int x1,y1,x2,y2;

    Property.Rect.GetCoords(x1,y1,x2,y2);
    y2 = y1;
    Property.Rect.SetCoords(x1,y1,x2,y2);
    Property.Style = 0;

    return true;
}

bool CProgress::OnClose()
{
    return true;
}

bool CProgress::OnPaint()
{
    CRect r;
    int x1,y1,x2,y2;
    float Index;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);


    MyScreen.SetColor(CPROGRESS_UNSEL);
    MyScreen.Fill(ACS_CKBOARD,x1,y1,x2,y2);

    if (Current >= Max)
	Index = 1;
    else if ((Max - Min) == 0 || Current <= Min)
	Index = 0;
    else {
	Index = (float)Current / (float)(Max - Min);
    }

    Index = Index * (x2 - x1 - 1);

    MyScreen.SetColor(CPROGRESS_SEL);
    MyScreen.Fill(ACS_BOARD,x1,y1,x1+Index-Min+1,y2);

    return true;
}

bool CProgress::OnGetFocus()
{
    return true;
}

bool CProgress::OnLoseFocus()
{
    return true;
}

bool CProgress::OnKeyDown(int Key,int Status)
{
    return true;
}

bool CProgress::OnLeftDown(CPoint p,int Status)
{
    return true;
}

bool CProgress::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CProgress::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CProgress::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CProgress::OnMove(CPoint p,int Status)
{
    return true;
}


void CProgress::SetRange(int a,int b,int c)
{
    Min = a;
    Max = b;
    Current = c;
}

int CProgress::GetCurrent()
{
    return Current;
}

void CProgress::SetCurrent(int n)
{
    if (Current >= Min && Current <= Max)
	Current = n;

    if (IsCreated)
	OnPaint();
}
