/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The radio box control of the interface library
 *
 */

#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "xinter.h"
#include "screen.h"

#ifdef X_DJGPP
#undef ACS_DIAMOND
#define ACS_DIAMOND 233
#endif

/*
 * For a radiobutton, Set the Property.Rect. Only the upperleft corner
 * counts. It sets the width and height itself. No styles need
 * to be set. But additems before creating an object and after setting the
 * rect. And dont add any
 * items after creating the object as its width and height wont be
 * recalculated.
 *
 * Returns no response
 *
 */


CRadio::CRadio() : Items(sizeof(char *))
{
    Select = 0;
}

CRadio::~CRadio()
{

}

bool CRadio::OnCreate()
{
    return true;
}

bool CRadio::OnPreCreate()
{
    int i;
    int x1,y1,x2,y2;
    
    Property.Rect.GetCoords(x1,y1,x2,y2);
    y2 = y1 + Items.GetTotal() - 1;
    Property.Rect.SetCoords(x1,y1,x2,y2);

    return true;
}

bool CRadio::OnClose()
{
    DeleteAll();

    return true;
}

bool CRadio::OnPaint()
{
    int x1,y1,x2,y2;
    CRect r;
    int i;
    char *Points;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    for (i=0;i<Items.GetTotal();i++) {
	Points = *(char **) Items.GetItem(i);

	MyScreen.SetColor(CRADIO_BCOL);
	MyScreen.PrintXY(x1,y1+i,"( )");

	MyScreen.SetColor(CRADIO_TCOL1);
	MyScreen.PutCharXY(x1+4,y1+i,*Points);
	MyScreen.SetColor(CRADIO_TCOL2);
	MyScreen.PrintXY(x1+5,y1+i,Points+1);
    }

    MyScreen.SetColor(CRADIO_SCOL);
    MyScreen.PutCharXY(x1+1,y1+Select,ACS_DIAMOND);

    return true;
}

bool CRadio::OnGetFocus()
{
    return true;
}

bool CRadio::OnLoseFocus()
{
    return true;
}

bool CRadio::OnKeyDown(int Key,int Status)
{
    if (Key == KEY_UP && Select > 0)
	Select--;
    else if (Key == KEY_DOWN && Select < Items.GetTotal()-1)
	Select++;

    OnPaint();

    return true;
}

bool CRadio::OnLeftDown(CPoint p,int Status)
{
    CRect r;
    int x1,y1,x2,y2;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    if (Select != p.y - y1) {
	Select = p.y - y1;
	OnPaint();
    }

    return true;
}

bool CRadio::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CRadio::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CRadio::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CRadio::OnMove(CPoint p,int Status)
{
    return true;
}


bool CRadio::SetSelect(int n)
{
    if (n >= 0 && n < Items.GetTotal()) 
	Select = n;

    return true;
}

int CRadio::GetSelect()
{
    return Select;
}

bool CRadio::AddItem(char *str)
{
    int x1,y1,x2,y2;
    char *Points;
    int Len;

    Len = strlen(str);
    Points = new char [Len+1];
    strcpy(Points,str);

    Items.AddTop(&Points);

    if (Len+4 > Property.Rect.GetWidth()) {
	Property.Rect.GetCoords(x1,y1,x2,y2);
	x2 = x1 + Len + 4 - 1;
	Property.Rect.SetCoords(x1,y1,x2,y2);
    }

    return true;
}

char *CRadio::GetItem(int n)
{
    char *Points;

    Points = *(char **) Items.GetItem(n);
    return Points;
}

bool CRadio::DeleteAll()
{
    int i;
    char *Points;

    for (i=0;i<Items.GetTotal();i++) {
	Points = *(char **) Items.GetItem(i);

	delete [] Points;
    }

    Items.DeleteAll();

    return true;
}
