/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The scrollbar control of the interface library
 *
 */

#include <stdlib.h>

#include "data.h"
#include "xinter.h"
#include "screen.h"
#include "xdraw.h"
#include "xcontrol.h"

#ifdef X_DJGPP
#undef ACS_DIAMOND
#define ACS_DIAMOND 233
#endif

/*
 * For a scrollbar, the user only needs to set the Property.Rect.
 * No other styles of properties need to be set. The scrollbar
 * returns Response on left down on its buttons or on Up or Down
 * key downs.
 *
 */

CScrollBar::CScrollBar()
{
    Min=Current=0;
    Max=100;
    showFocus = false;
}

CScrollBar::~CScrollBar()
{

}

bool CScrollBar::OnCreate()
{
    return true;
}

bool CScrollBar::OnPreCreate()
{
    int x1,y1,x2,y2;

    Property.Rect.GetCoords(x1,y1,x2,y2);
    x2 = x1;
    Property.Rect.SetCoords(x1,y1,x2,y2);
    Property.Style = 0;

    Property.Rect.SetMinWidth(1);
    Property.Rect.SetMinHeight(y2-y1+1);

    return true;
}

bool CScrollBar::OnClose()
{
    return true;
}

bool CScrollBar::OnPaint()
{
    CRect r;
    int x1,y1,x2,y2;
    float Index;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);


    MyDraw.VLine(y1,y2,x1);
    MyScreen.SetColor((showFocus) ? CSCROLL_PCOL : CSCROLL_BCOL);
    MyScreen.PutCharXY(x1,y1,ACS_UARROW);
    MyScreen.PutCharXY(x1,y2,ACS_DARROW);

    if ((Max-Min) != 0) {
	Index = (float)Current / (float)(Max - Min);
	Index = Index * (y2 - y1 - 2);
    }
    else
	Index = 0;

    MyScreen.SetColor(CSCROLL_PCOL);
    MyScreen.PutCharXY(x1,(int)(y1+Index-Min+1),ACS_DIAMOND);

    return true;
}

bool CScrollBar::OnGetFocus()
{
    if (showFocus) {
        CRect r;
        int x1,y1,x2,y2;

        r = MapToScreen(Property.Rect,this);
        r.GetCoords(x1,y1,x2,y2);

        MyScreen.SetColor(CSCROLL_PCOL);
        MyScreen.PutCharXY(x1,y1,ACS_UARROW);
        MyScreen.PutCharXY(x1,y2,ACS_DARROW);
    }

    return true;
}

bool CScrollBar::OnLoseFocus()
{
    if (showFocus) {
        CRect r;
        int x1,y1,x2,y2;

        r = MapToScreen(Property.Rect,this);
        r.GetCoords(x1,y1,x2,y2);

        MyScreen.SetColor(CSCROLL_BCOL);
        MyScreen.PutCharXY(x1,y1,ACS_UARROW);
        MyScreen.PutCharXY(x1,y2,ACS_DARROW);
    }

    return true;
}

bool CScrollBar::OnKeyDown(int Key,int Status)
{
    int OCur;

    OCur = Current;

    if (Key == KEY_UP || Key == KEY_DOWN) {
	if (Key == KEY_DOWN && Current<Max) Current++;
	if (Key == KEY_UP && Current>Min) Current--;

	if (OCur == Current) 
	    return false;

	OnPaint();

	if (Property.Response)
	    Property.Response->OnCommand(Property.Handle,RE_KEYDOWN,
					 CPoint(0,0),Key,Status);
    }

    return true;
}

bool CScrollBar::OnLeftDown(CPoint p,int Status)
{
    CRect r;
    int x1,y1,x2,y2;
    int OCur;

    OCur = Current;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    if (p.x == x1 && (p.y == y1 || p.y == y2)) {
	if (p.y == y1 && Current>Min) Current--;
	if (p.y == y2 && Current<Max) Current++;
    }


    if (OCur == Current)
	return false;

    OnPaint();

    if (Property.Response)
	Property.Response->OnCommand(Property.Handle,RE_LEFTDOWN,
				     p,0,Status);

    return true;
}

bool CScrollBar::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CScrollBar::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CScrollBar::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CScrollBar::OnMove(CPoint p,int Status)
{
    return true;
}

void CScrollBar::SetRange(int min,int max,int cur)
{
    Min = min;
    Max = max;
    Current = cur;
}

int CScrollBar::GetCurrent()
{
    return Current;
}

void CScrollBar::SetCurrent(int n)
{
    Current = n;
}

bool CScrollBar::ShowFocus(bool val)
{
    showFocus = val;
    return true;
}
