/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The slider control of the interface library
 *
 */

#include <stdlib.h>

#include "data.h"
#include "xinter.h"
#include "xdraw.h"
#include "xcontrol.h"


#ifdef X_DJGPP
#undef ACS_BULLET
#define ACS_BULLET 254
#endif

/*
 * For a slider, the user only needs to set the Property.Rect.
 * No other styles of properties need to be set. The scrollbar
 * returns Response on left down when its current changes or on 
 * left or right key downs.
 *
 */


CSlider::CSlider()
{
    Min = 0;
    Max = 100;
    Current = 0;
}

CSlider::~CSlider()
{

}

bool CSlider::OnCreate()
{
    return true;
}

bool CSlider::OnPreCreate()
{
    int x1,y1,x2,y2;

    Property.Rect.GetCoords(x1,y1,x2,y2);
    y2 = y1;
    Property.Rect.SetCoords(x1,y1,x2,y2);
    Property.Style = 0;

    return true;
}

bool CSlider::OnClose()
{
    return true;
}

bool CSlider::OnPaint()
{
    CRect r;
    int x1,y1,x2,y2;
    float Index;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);


    MyDraw.HLine(x1+1,x2-1,y1,CSLIDER_HLINE);
    MyScreen.SetColor(CSLIDER_ACOL);
    MyScreen.PutCharXY(x1,y1,ACS_LARROW);
    MyScreen.PutCharXY(x2,y1,ACS_RARROW);

    Index = (float)Current / (float)(Max - Min);
    Index = Index * (x2 - x1 - 2);

    MyScreen.SetColor(CSLIDER_SCOL);
    MyScreen.PutCharXY(x1+Index-Min+1,y1,ACS_BULLET);

    return true;
}

bool CSlider::OnGetFocus()
{
    CRect r;
    int x1,y1,x2,y2;
    float Index;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    Index = (float)Current / (float)(Max - Min);
    Index = Index * (x2 - x1 - 2);

    MyScreen.SetColor(CSLIDER_SCOL);
    MyScreen.PutCharXY(x1+Index-Min+1,y1,ACS_BULLET);

    return true;
}

bool CSlider::OnLoseFocus()
{
    CRect r;
    int x1,y1,x2,y2;
    float Index;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    Index = (float)Current / (float)(Max - Min);
    Index = Index * (x2 - x1 - 2);

    MyScreen.SetColor(CSLIDER_USCOL);
    MyScreen.PutCharXY(x1+Index-Min+1,y1,ACS_BULLET);

    return true;
}

bool CSlider::OnKeyDown(int Key,int Status)
{
    int OCur;

    OCur = Current;

    if (Key == KEY_LEFT || Key == KEY_RIGHT) {
	if (Key == KEY_RIGHT && Current<Max) Current++;
	if (Key == KEY_LEFT && Current>Min) Current--;

	if (OCur == Current) 
	    return false;

	OnPaint();

	if (Property.Response)
	    Property.Response->OnCommand(Property.Handle,RE_KEYDOWN,
					 CPoint(0,0),Key,Status);
    }

    return true;
}

bool CSlider::OnLeftDown(CPoint p,int Status)
{
    CRect r;
    int x1,y1,x2,y2;
    int OCur;
    float Index;

    OCur = Current;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    if (p.y == y1 && (p.x == x1 || p.x == x2)) {
	if (p.x == x1 && Current>Min) Current--;
	if (p.x == x2 && Current<Max) Current++;
    }
    else if (p.y == y1 && (p.x > x1 && p.x < x2)) {
	Index = (float)(p.x - x1) / (float)(x2 - x1 - 1);
	Index = Index * (Max - Min);
	Current = (int)Index;
    }

    if (OCur == Current)
	return false;

    OnPaint();

    if (Property.Response)
	Property.Response->OnCommand(Property.Handle,RE_LEFTDOWN,
				     p,0,Status);

    return true;
}

bool CSlider::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CSlider::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CSlider::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CSlider::OnMove(CPoint p,int Status)
{
    return true;
}


void CSlider::SetRange(int a,int b,int c)
{
    Min = a;
    Max = b;
    Current = c;
}

int CSlider::GetCurrent()
{
    return Current;
}

void CSlider::SetCurrent(int n)
{
    if (Current >= Min && Current <= Max)
	Current = n;
}
