/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The textarea control of the interface library
 *
 */

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "data.h"
#include "xinter.h"
#include "xdraw.h"
#include "screen.h"
#include "xcontrol.h"

#if defined(X_UNIX) || defined(X_DJGPP)
#undef KEY_ENTER
#define KEY_ENTER 10
#endif

#ifdef X_DJGPP
#undef KEY_BACKSPACE
#define KEY_BACKSPACE 8
#endif

/*
 * For this class you need only to set the Property.Rect. A charset maybe
 * set. Use getline and setline to get and set text in the text area.
 * The area becomes scrollable if the region grows out of its size.
 *
 */


CTextArea::CTextArea()
{
    CurX = CurY = 0;
    MaxLines = 10;

    CharSet = NULL;

    Scroll = 0;
    Scrollable = false;
    ScrollHandle = -1;
}

CTextArea::~CTextArea()
{
    CurX = CurY = Scroll = 0;
}

bool CTextArea::OnCreate()
{
    CWindow::OnCreate();

    return true;
}

bool CTextArea::OnPreCreate()
{
    int i= KEY_ENTER;
    int x1,y1,x2,y2;

    Length = Property.Rect.GetWidth() - 5;
    Data.Initialize(Length+1);
    
    if (CharSet == NULL) {
	CharSet = new char [strlen(CTEXTAREA_ALPHANUM)+1];
	strcpy(CharSet,CTEXTAREA_ALPHANUM);
    }

    Clear();

    CWindow::OnPreCreate();

    return true;
}

bool CTextArea::OnClose()
{
    CWindow::OnClose();

    Data.DeleteAll();
    if (CharSet)
	delete [] CharSet;

    CurX = CurY = 0;
    CharSet = NULL;
    Scroll = 0;
    Scrollable = false;
    ScrollHandle = -1;

    return true;
}

bool CTextArea::OnPaint()
{
    int x1,y1,x2,y2;
    CRect r;
    int i,j;
    int h;
    char *say;

    CWindow::OnPaint();
    
    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    h = r.GetHeight() - 2;

    
    MyScreen.SetColor(CTEXTAREA_COL);

    for (i=0;i<h;i++) {
	if (Scroll+i >= Data.GetTotal())
	    break;

	say = (char *) Data.GetItem(Scroll+i);

	MyScreen.SetCursor(x1+2,y1+i+1);

	for (j=0;j<Length;j++) {
	    if (!say[j])
		break;

	    MyScreen.PutChar(say[j]);
	}

    }
   
    MyScreen.SetCursor(x1+CurX+2,y1+CurY-Scroll+1);

    return true;
}

bool CTextArea::OnGetFocus()
{
    int x1,y1,x2,y2;
    CRect r;
    
    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    
    CWindow::OnGetFocus();
    MyScreen.SetCursor(x1+CurX+2,y1+CurY-Scroll+1);
  
    return true;
}

bool CTextArea::OnLoseFocus()
{
    CWindow::OnLoseFocus();

    return true;
}

bool CTextArea::OnKeyDown(int Key,int Status)
{
    int Total;
    char *line;
    int Height;


    Total = Data.GetTotal();
    Height = Property.Rect.GetHeight() - 2;
    line = (char *) Data.GetItem(CurY);

    switch (Key) {
    case KEY_LEFT:
	if (CurX != 0)
	    CurX--;

	break;

    case KEY_RIGHT:
	if (CurX != Length-1)
	    CurX++;

	break;

    case KEY_HOME:
	CurX = 0;
	break;

    case KEY_END:
	CurX = Length-1;
	break;

    case KEY_DC:
	Delch(CurX);
	break;

    case KEY_BACKSPACE:
	if (CurX != 0) {
	    CurX--;
	    Delch(CurX);
	}
	break;

    case KEY_DOWN:
	if (CurY < Total-1) {
	    if (CurY-Scroll == Height-1) Scroll++;
	    CurY++;
	    AddScroll();
	}

	break;

    case KEY_UP: 
	if (CurY > 0) {
	    if (CurY == Scroll) Scroll--;
	    CurY--;
	    AddScroll();
	}

	break;

    case KEY_ENTER:
	if (Total<MaxLines) {
	    CurX = 0;
	    if (CurY-Scroll == Height-1) Scroll++;
	    CurY++;
	    CreateLine();
	    AddScroll();
	}
	
	break;

    default:
	if (strchr(CharSet,Key)) {
	    Insch();
	    line[CurX] = (char) Key;
	    CurX++;
	    if (CurX == Length) {
		if (Total<MaxLines) {
		    CurX = 0;
		    if (CurY-Scroll == Height-1) Scroll++;
		    CurY++;
		    CreateLine();
		    AddScroll();
		}
		else
		    CurX--;
	    }
	}
    }

    OnPaint();

    return true;
}

bool CTextArea::OnLeftDown(CPoint p,int Status)
{
    CWindow::OnLeftDown(p,Status);

    return true;
}

bool CTextArea::OnRightDown(CPoint p,int Status)
{
    CWindow::OnRightDown(p,Status);

    return true;
}

bool CTextArea::OnLeftUp(CPoint p,int Status)
{
    CWindow::OnLeftUp(p,Status);

    return true;
}

bool CTextArea::OnRightUp(CPoint p,int Status)
{
    CWindow::OnRightUp(p,Status);

    return true;
}

bool CTextArea::OnMove(CPoint p,int Status)
{
    CWindow::OnMove(p,Status);

    return true;
}

bool CTextArea::OnCommand(int Handle,long int Event,CPoint p,int Key,
			  int Status)
{
    CRect r;
    int x1,y1,x2,y2;
    int Height;
    CScrollBar *MyBar;
    int Current;


    if (!Scrollable || Handle != ScrollHandle)
	return true;

    if (Event == RE_LEFTDOWN) {
	MyBar = (CScrollBar *) GetControl(ScrollHandle);
	r = MapToScreen(MyBar->Property.Rect,MyBar);
	r.GetCoords(x1,y1,x2,y2);

	Height = Property.Rect.GetHeight() - 2;

	if (p.y == y1) {
	    if (CurY == Scroll) Scroll--;
	    CurY--;
	}
	else if (p.y == y2) {
	    if (CurY-Scroll == Height-1) Scroll++;
	    CurY++;
	}

	OnPaint();

    }

    return true;
}



bool CTextArea::SetCharSet(char *str)
{
    if (CharSet)
	delete [] CharSet;

    CharSet = new char [strlen(str)+1];
    strcpy(CharSet,str);

    return true;
}

int CTextArea::TotalLines()
{
    return Data.GetTotal();
}

int CTextArea::GetLength()
{
    return Length;
}

bool CTextArea::SetMaxLines(int n)
{
    MaxLines = n;
}


bool CTextArea::GetLine(int n,char *Buffer)
{
    char *Points;
    int Len;
    int i;

    Points = (char*) Data.GetItem(n);
    strcpy(Buffer,Points);

    for (i=Length-1;i>=0;i--) {
	if (Buffer[i] == ' ')
	    Buffer[i] = '\0';
	else
	    break;
    }

    return true;
}

bool CTextArea::SetLine(int n,char *str)
{
    char *Points;
    int i,j;

    Points = (char *) Data.GetItem(n);

    if (!Points)
	return false;

    memset(Points,' ',Length);
    Points[Length] = '\0';

    for (i=0;i<Length;i++) {
	if (*str == '\0')
	    break;

	*Points++ = *str++;
    }

    return true;
}

bool CTextArea::AddLine(char *str)
{
    char *Points;

    Points = new char [Length + 1];
    memset(Points,' ',Length);

    Points[Length] = '\0';
    
    for (int i=0;i<Length;i++) {
	if (!str[i]) break;
	Points[i] = str[i];
    }

    Data.AddTop(Points);
    delete [] Points;

    return true;
}

bool CTextArea::Clear()
{
    Data.DeleteAll();
    Scroll = CurX = CurY = 0;
    CreateLine();
    AddScroll();
}


bool CTextArea::CreateLine()
{
    char *Points;

    Points = new char [Length + 1];
    memset(Points,' ',Length);

    Points[Length] = '\0';
    
    Data.InsertItem(CurY,Points);
    delete [] Points;

    return true;
}

void CTextArea::Delch(int n)
{
    int i;
    char *Buffer;

    Buffer = (char*) Data.GetItem(CurY);

    for (i=n+1;i<Length;i++) {
	Buffer[i-1] = Buffer[i];
    }

    Buffer[Length-1] = ' ';
}

void CTextArea::Insch()
{
    int i;
    char *Buffer;

    Buffer = (char*) Data.GetItem(CurY);

    for (i=Length-1;i>CurX;i--) {
	Buffer[i] = Buffer[i-1];
    }

    Buffer[CurX] = ' ';
}

bool CTextArea::AddScroll()
{
    int w,h;
    CScrollBar *MyBar;

    w = Property.Rect.GetWidth();
    h = Property.Rect.GetHeight();

    if (Scrollable) {
	MyBar = (CScrollBar *) GetControl(ScrollHandle);
	MyBar->SetRange(0,Data.GetTotal()-1,CurY);
    }
    else if (Data.GetTotal() > h-2) {
	MyBar = new CScrollBar;
	MyBar->Property.SetCoords(w-2,1,w-2,h-2);
	MyBar->SetRange(0,Data.GetTotal()-1,CurY);
	MyBar->Property.Response = this;
	
	ScrollHandle = AddControl(MyBar);
	Scrollable = true;
    }

    return true;
}

bool CTextArea::IsEmpty(int line)
{
    char *Buffer = new char [Length+1];
    GetLine(line,Buffer);

    char *ptr = Buffer;

    while (*ptr) {
	if (!isspace(*ptr)) { delete [] Buffer; return false; }
	ptr++; 
    }
    delete [] Buffer;
    return true;
}
