/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The textbox control of the interface library
 *
 */

#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "xinter.h"
#include "screen.h"
#include "xcontrol.h"


/*
 * For this textbox you only need to set Property.Rect and also set the
 * property.style to cwin_border. Then call settext to display a word
 * wrapped para of text. Automatically inserts a scrollbar if the text
 * runs out of the box.
 * SetText can be called before and after creating the window. But it
 * MUST be called after setting property.rect (also for cattext)
 *
 */

CTextBox::CTextBox() : Data(sizeof(char*))
{
    Scrollable = false;
    ScrollHandle = -1;
    Scroll = 0;
    RawData = NULL;
}

CTextBox::~CTextBox()
{
    if (RawData)
        delete [] RawData;
}

bool CTextBox::OnCreate()
{
    CWindow::OnCreate();
    return true;
}

bool CTextBox::OnPreCreate()
{
    int w,h;
    int x1,y1,x2,y2;
    CRect r;

    h = Property.Rect.GetHeight();
    w = Property.Rect.GetWidth();

    if (Data.GetTotal() > h-2) {
	Scrollable = true;

	CScrollBar *MyBar = new CScrollBar;
        MyBar->Property.SetCoords(w-2,1,w-2,h-2);
        MyBar->SetRange(0,Data.GetTotal()-(h-2),Scroll);
        MyBar->Property.Response = this;

	ScrollHandle = AddControl(MyBar);
    }

    CWindow::OnPreCreate();

    return true;
}

bool CTextBox::OnClose()
{
    int i;
    char *Points;

    CWindow::OnClose();

    for (i=0;i<Data.GetTotal();i++) {
	Points = *(char **) Data.GetItem(i);
	delete [] Points;
    }

    Data.DeleteAll();

    return true;
}

bool CTextBox::OnPaint()
{
    int x1,y1,x2,y2;
    CRect r;
    int i;
    int Width,Height;
    char *Points;

    CWindow::OnPaint();

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    Width = Property.Rect.GetWidth() - 5;
    Height = Property.Rect.GetHeight() - 2;

    MyScreen.SetColor(CTEXTBOX_COL);

    for (i=0;i<Height;i++) {
	if (Scroll+i >= Data.GetTotal())
	    break;

	Points = *(char**) Data.GetItem(Scroll+i);

	MyScreen.SetCursor(x1+2,y1+1+i);
	MyScreen.Print(Points);
    }

    return true;
}

bool CTextBox::OnGetFocus()
{
    CWindow::OnGetFocus();
    return true;
}

bool CTextBox::OnLoseFocus()
{
    CWindow::OnLoseFocus();
    return true;
}

bool CTextBox::OnKeyDown(int Key,int Status)
{
    int h;
    CScrollBar *MyBar;
    MyBar = (CScrollBar*) GetControl(ScrollHandle);

    h = Property.Rect.GetHeight() - 2;

    if (Scrollable) {
	if (Key == KEY_UP && Scroll>0) {
	    Scroll--;
	}
	else if (Key == KEY_DOWN && Scroll<Data.GetTotal()-h){
	    Scroll++;
	}

	MyBar->SetCurrent(Scroll);
	OnPaint();
    }

    return true;
}

bool CTextBox::OnLeftDown(CPoint p,int Status)
{
    CWindow::OnLeftDown(p,Status);
    return true;
}

bool CTextBox::OnRightDown(CPoint p,int Status)
{
    return true;
}

bool CTextBox::OnLeftUp(CPoint p,int Status)
{
    return true;
}

bool CTextBox::OnRightUp(CPoint p,int Status)
{
    return true;
}

bool CTextBox::OnMove(CPoint p,int Status)
{
    return true;
}

bool CTextBox::OnCommand(int Handle,long int Event,CPoint p,int Key,
		       int Status)
{
    CRect r;
    int x1,y1,x2,y2;
    int Height;
    CScrollBar *MyBar;
    int Current;


    if (!Scrollable || Handle != ScrollHandle)
	return true;

    if (Event == RE_LEFTDOWN) {
	MyBar = (CScrollBar *) GetControl(ScrollHandle);
	r = MapToScreen(MyBar->Property.Rect,MyBar);
	r.GetCoords(x1,y1,x2,y2);

	Height = Property.Rect.GetHeight() - 2;

	if (p.y == y1) {
	    Scroll--;
	}
	else if (p.y == y2) {
	    Scroll++;
	}

	OnPaint();
    }

    return true;
}


bool CTextBox::SetText(char *Text)
{
    char *Points;
    int i;

    if (RawData) delete [] RawData;
    RawData = NULL;

    /*
    for (i=0;i<Data.GetTotal();i++) {
	Points = *(char **) Data.GetItem(i);
	delete [] Points;
    }

    Data.DeleteAll();
    */

    CatText(Text);

    return true;
}

bool CTextBox::CatText(char *Text)
{
    char *Points;
    int i,j;

    int Width = Property.Rect.GetWidth() - 5;
    int Len,TotalLen;

    char *Buffer;
    char *Source,*Dest;

    for (i=0;i<Data.GetTotal();i++) {
	Points = *(char **) Data.GetItem(i);
	delete [] Points;
    }

    Data.DeleteAll();

    if (!RawData) {
        RawData = new char[strlen(Text)+1];
        strcpy(RawData, Text);
    }
    else {
        char *tmp = new char[strlen(RawData)+strlen(Text)+1];
	strcpy(tmp, RawData);
	strcat(tmp, Text);
	delete [] RawData;
	RawData = tmp;
    }
    
    Source = RawData;
    bool fdone = false;

    while (!fdone) {
	Buffer = new char [Width+1];
	Data.AddTop(&Buffer);

	Dest = Buffer;
	TotalLen = 0;

	while (1) {
	    Len = WordLength(Source);

	    if (Len == 0) {
		*Dest = '\0';
		fdone = true;
		break;
	    }
	    else if (Len == -1) {
		*Dest = '\0';
		*Source++;
		break;
	    }

	    if (Len > Width) {
		for (j=0;j<Width-TotalLen;j++) {
		    *Dest++ = *Source++;
		}
		*Dest = '\0';
		break;
	    }
	    else if (Len+TotalLen > Width) {
		if (*Source == ' ')
		    Source++;

		*Dest = '\0';
		break;
	    }
	    else {
		TotalLen += Len;
		
		for (j=0;j<Len;j++) {
		    *Dest++ = *Source++;
		}
	    }
	}
    }

    int h = Property.Rect.GetHeight();
    int w = Property.Rect.GetWidth();

    /* Now add scrollbar if needed */
    if (Data.GetTotal() > h-2) {
        Scroll = Data.GetTotal()-(h-2);

        if (!Scrollable) {
            if (Data.GetTotal() > h-2) {
                Scrollable = true;

    	    CScrollBar *MyBar = new CScrollBar;
                MyBar->Property.SetCoords(w-2,1,w-2,h-2);
                MyBar->SetRange(0,Data.GetTotal()-(h-2),Scroll);
                MyBar->Property.Response = this;

                ScrollHandle = AddControl(MyBar);
            }
        }
        else {
            CScrollBar *MyBar = (CScrollBar*) GetControl(ScrollHandle);
            MyBar->SetRange(0,Data.GetTotal()-(h-2),Scroll);
        }
    }
    else Scroll = 0;


    return true;
}

bool CTextBox::ReAlign()
{
    if (RawData) {
        char *tmp = new char[strlen(RawData)+1];
        strcpy(tmp, RawData);
        SetText(tmp);
        delete [] tmp;
    }

    int h = Property.Rect.GetHeight();
    int w = Property.Rect.GetWidth();

    if (Scrollable) {
        CScrollBar *MyBar = (CScrollBar*) GetControl(ScrollHandle);
        MyBar->Property.SetCoords(w-2,1,w-2,h-2);
    }

    return true;
}

char *CTextBox::GetLine(int n)
{
    char *Points;
    Points = *(char **) Data.GetItem(n);

    return Points;
}

int CTextBox::TotalLines()
{
    return Data.GetTotal();
}

int CTextBox::WordLength(char *from)
{
    int Count = 0;

    if (*from == '\n') return -1;

    while (*from == ' ') {
	from++;
	Count++;
    }

    while (*from != 0 && *from != ' ' && *from != '\n') {
	from++;
	Count++;
    }

    return Count;
}
