/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - Some utility classes for the user generated
 *          using the interface library.
 *
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>

#include "data.h"
#include "xinter.h"
#include "xcontrol.h"
#include "xutil.h"

#if defined(X_UNIX) || defined(X_DJGPP)
#undef KEY_ENTER
#define KEY_ENTER 10
#endif


CFileDialog::CFileDialog(int s,int m)
{
    SigHandle = s;
    Mode = m;
}
 
bool CFileDialog::OnPreCreate() 
{
    Property.SetStyle(CWIN_MOVE|CWIN_BORDER|CWIN_TITLE|CWIN_DIALOG|CWIN_TABCYCLE);
    Property.SetTitle("File Dialog");
    Property.SetCoords(10,4,56,20);


    char mycwdir[CFILEDIALOG_FMAX];
    getcwd(mycwdir,CFILEDIALOG_FMAX);

    CEditBox *MyEdit = new CEditBox;
    MyEdit->Property.SetCoords(2,2,44,2);
    MyEdit->Property.Response = this;
    MyEdit->SetLength(CFILEDIALOG_FMAX);
    MyEdit->SetCharSet(CFILEDIALOG_FCHAR);
    MyEdit->SetText(mycwdir);
    EBox = AddControl(MyEdit);

    CListBox *MyList = new CListBox;
    MyList->Property.SetCoords(2,3,44,12);
    MyList->Property.SetStyle(CWIN_BORDER);
    MyList->Property.Response = this;
    MyList->SetFollow(false);
    MyList->SetScrollable(true);
    LBox = AddControl(MyList);

    CButton *OkButton = new CButton;
    OkButton->Property.SetCoords(15,14,1,1);
    OkButton->Property.SetTitle("Okay");
    OkButton->Property.Response = this;
    Ok = AddControl(OkButton);

    CButton *CanButton = new CButton;
    CanButton->Property.SetCoords(23,14,1,1);
    CanButton->Property.SetTitle("Cancel");
    CanButton->Property.Response = this;
    Cancel = AddControl(CanButton);


    UpdateList();

    return true;
}

bool CFileDialog::OnCommand(int Handle,long int Event,CPoint p,
			    int Key,int Status)
{
    char buffer[CFILEDIALOG_FMAX+1];
    void *data[5];
    char *ptr;
    struct stat MyStat;
   
    CEditBox *Eb;
    CListBox *Lb;

    if (Handle == Cancel) {
	Control.CloseWindow();
	Control.RepaintAll();
	return true;
    }
    else if (Handle == Ok || (Handle == EBox && Key == KEY_ENTER) ||
	     (Handle == LBox && Key == KEY_ENTER)) {

	int checks = 0;

	if (!Property.Response) return true;

	Eb = (CEditBox *) GetControl(EBox);
	Eb->GetText(buffer);

	if (!access(buffer,F_OK)) {
	    if (stat(buffer,&MyStat)) return true;

	    if (S_ISDIR(MyStat.st_mode)) {
                if (Handle == Ok && Mode == CFILEDIALOG_DOPEN)
                    checks = 1;
                else {
                    chdir(buffer);

                    if (getcwd(buffer,CFILEDIALOG_FMAX) == NULL) return true;
                    Eb->SetText(buffer);
                    UpdateList();

                    OnPaint();
                }
	    }
	    else if (Mode != CFILEDIALOG_DOPEN)
		checks = 1;
	}
	else if (Mode == CFILEDIALOG_FSAVE)
	    checks = 1;

	if (checks) {
	    ptr = new char [Eb->GetLength()+1];
	    data[0] = (void *) ptr;
	    strcpy((char*)data[0],buffer);
	    Property.Response->OnSignal(SigHandle,CSIG_FILEDIALOG,data);
	    delete [] ptr;

	    Control.CloseWindow();
	    Control.RepaintAll();
	}

    }
    else if (Handle == LBox) {
	if (getcwd(buffer,CFILEDIALOG_FMAX) == NULL) return true;
	if (buffer[strlen(buffer)-1] != '/') strcat(buffer,"/");

	Lb = (CListBox *) GetControl(LBox);
	ptr = Lb->GetItem();

	if (ptr != NULL) {
	    Eb = (CEditBox *) GetControl(EBox);

	    strcat(buffer,ptr);
	    Eb->SetText(buffer);

	    OnPaint();
	}
    }

    return true;
}

bool CFileDialog::UpdateList()
{
    DIR *MyDir;
    struct dirent *MyDirent;
    struct stat MyStat;
    CListBox *MyList;
    char buffer[CFILEDIALOG_FMAX+1];

    MyList = (CListBox *) GetControl(LBox);
    MyList->DeleteAll();

    if (getcwd(buffer,CFILEDIALOG_FMAX) == NULL) return false;
    MyDir = opendir(buffer);
    if (MyDir == NULL) return false;

    while (MyDirent = readdir(MyDir)) {
	strcpy(buffer,MyDirent->d_name);
	if (stat(buffer,&MyStat)) continue;

	if (S_ISDIR(MyStat.st_mode)) strcat(buffer,"/");

	MyList->AddItem(buffer);
    }

    closedir(MyDir);

    return true;
}
