/*
 * wcon v0.5 GUI library for the console
 * Written by Fakhir Ansari <ansari89@yahoo.com>
 *
 * Module - The window class of the interface library.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "data.h"
#include "screen.h"
#include "xdraw.h"
#include "xinter.h"
#include "xcontrol.h"


/*************************************************************************/

CWinProperty::CWinProperty() {
    Style = 0;
    Title = NULL;
    Parent = NULL;
    Response = NULL;
    Handle = 0;
    Visible = true;

    Rect.SetMinWidth(10);
    Rect.SetMinHeight(3);
}

CWinProperty::~CWinProperty() {
    if (Title)
	delete [] Title;
}

void CWinProperty::SetTitle(char *s) {
    Title = new char [strlen(s)+1];
    strcpy(Title,s);
}

void CWinProperty::SetStyle(long int wm)
{
    Style = wm;
}

void CWinProperty::SetCoords(int x1,int y1,int x2,int y2)
{
    Rect.SetCoords(x1,y1,x2,y2);
}

/**********************************************************************/

CWindow::CWindow() : ControlList(sizeof(CWindow*))
{
    IsCreated = 0;
    IsMoving = 0;
    IsResizing = 0;

    ControlCount = 0;
    CurrentControl = -1;
}


CWindow::~CWindow()
{

}


bool CWindow::OnCreate()
{
    CWindow *Points;
    int i;

    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow **) ControlList.GetItem(i);
	Points->OnPreCreate();
    }
    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow **) ControlList.GetItem(i);
	Points->OnCreate();
    }

    IsCreated = 1;
    return true;
}

bool CWindow::OnPreCreate()
{
    return true;
}

bool CWindow::OnClose()
{
    CWindow *Points;
    int i;

    if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnLoseFocus();
    }
    OnLoseFocus();

    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow **) ControlList.GetItem(i);

	Points->OnClose();
	delete Points;
    }

    ControlList.DeleteAll();

    ControlCount = 0;
    CurrentControl = -1;


    return true;
}

bool CWindow::OnPaint()
{
    CWindow *Points;
    int i;
    int x1,x2,y1,y2;
    CRect r;
    CRect r2;

    if (!(Property.Visible))
	return true;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);
    MyScreen.Clear(x1,y1,x2,y2);

    if (Property.Style & CWIN_BORDER)
	MyDraw.DrawKewlBox(CRect(x1,y1,x2,y2));

    if (Property.Style & CWIN_TITLE)
	MyDraw.TextCenter(CPoint(x1,y1),CPoint(x2,y2),
			  Property.Title,CWIN_TITLECOL);

    if (Property.Style & CWIN_CLOSEB) {
	MyScreen.SetCursor(x2 - 1,y1);
	MyScreen.SetColor(CWIN_CROSSCOL);
	MyScreen.Print("X");
    }

    for (i=ControlList.GetTotal()-1;i>=0;i--) {
	Points = *(CWindow **) ControlList.GetItem(i);

	if (!Points->Property.Visible)
	    continue;

	r = MapToScreen(Points->Property.Rect,Points);
	r2 = MapToScreen(Property.Rect,this);
	if (r2.IsIn(r)) {
	    Points->OnPaint();
	    Points->OnLoseFocus();
	}
    }

    if (CurrentControl!=-1) {
	Points = GetControl(CurrentControl);
	if (Points->Property.Visible)
	    Points->OnGetFocus();
	else
	    CurrentControl = -1;
    }

    return true;
}

bool CWindow::OnGetFocus()
{
    CWindow *Points;
    int x1,x2,y1,y2;
    CRect r;

    if (!(Property.Visible))
	return true;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    if (Property.Style & CWIN_BORDER)
	MyDraw.DrawKewlBox(CRect(x1,y1,x2,y2));

    if (Property.Style & CWIN_TITLE)
	MyDraw.TextCenter(CPoint(x1,y1),CPoint(x2,y2),
			  Property.Title,CWIN_TITLECOL);

    if (Property.Style & CWIN_CLOSEB) {
	MyScreen.SetCursor(x2 - 1,y1);
	MyScreen.SetColor(CWIN_CROSSCOL);
	MyScreen.Print("X");
    }

    if (CurrentControl!=-1) {
	Points = GetControl(CurrentControl);
	if (Points->Property.Visible)
	    Points->OnGetFocus();
	else
	    CurrentControl = -1;
    }

    return true;
}

bool CWindow::OnLoseFocus()
{
    int x1,x2,y1,y2;
    CWindow *Points;
    CRect r;

    if (!(Property.Visible))
	return true;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);


    if (Property.Style & CWIN_TITLE)
	MyDraw.TextCenter(CPoint(x1,y1),CPoint(x2,y2),
			  Property.Title,CWIN_DTITLECOL);

    if (Property.Style & CWIN_CLOSEB) {
	MyScreen.SetCursor(x2 - 1,y1);
	MyScreen.SetColor(CWIN_DCROSSCOL);
	MyScreen.Print("X");
    }

    if (CurrentControl!=-1) {
	Points = GetControl(CurrentControl);
	Points->OnLoseFocus();
    }

    return true;
}


bool CWindow::OnKeyDown(int Key,int Status)
{
    int i;
    CWindow *Points;

    if (Property.Style & CWIN_TABCYCLE && Key == CWIN_KEY_CYCLE) {
        NextControl();
    }
    else if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnKeyDown(Key,Status);
    }

    return true;
}

bool CWindow::OnLeftDown(CPoint p,int Status)
{
    CWindow *Points;
    CRect r;
    int x1,y1,x2,y2;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    
    if ((Property.Style & CWIN_MOVE) && (p.y==y1) && (p.x!=x2-1)) {
	IsMoving = true;
	mov_down_x = p.x;
	mov_down_y = p.y;
    }

    if ((Property.Style & CWIN_RESIZE) && (p.y==y2) && (p.x==x2)) {
	IsResizing = true;
	res_down_x = p.x;
	res_down_y = p.y;
    }

    ControlFromPoint(p);

    if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnLeftDown(p,Status);
    }

    return true;
}

bool CWindow::OnRightDown(CPoint p,int Status)
{
    CWindow *Points;

    ControlFromPoint(p);

    if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnRightDown(p,Status);
    }

    return true;
}

bool CWindow::OnLeftUp(CPoint p,int Status)
{
    CWindow *Points;

    CRect r;
    int x1,y1,x2,y2;

    r = MapToScreen(Property.Rect,this);
    r.GetCoords(x1,y1,x2,y2);

    if (Property.Style & CWIN_CLOSEB) {
	if (p.x == x2 - 1 && p.y == y1) {
	    Control.CloseWindow();
	    Control.RepaintAll();
	    return true;
	}
    }

    if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnLeftUp(p,Status);
    }

    if (IsMoving)
	IsMoving = false;

    if (IsResizing) 
	IsResizing = false;

    return true;
}

bool CWindow::OnRightUp(CPoint p,int Status)
{
    CWindow *Points;

    if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnRightUp(p,Status);
    }

    return true;
}

bool CWindow::OnMove(CPoint p,int Status)
{
    int i;
    int Delta_X, Delta_Y;
    CWindow *Points;
    CRect r;
    CRect r2;

    if (CurrentControl != -1) {
	Points = GetControl(CurrentControl);
	Points->OnMove(p,Status);
    }

    if (IsMoving) {
	Delta_X = p.x - mov_down_x;
	Delta_Y = p.y - mov_down_y;

	mov_down_x = p.x;
	mov_down_y = p.y;

	Property.Rect.MoveD(Delta_X,Delta_Y);
	Control.RepaintAll();

    }

    if (IsResizing) {
	Delta_X = p.x - res_down_x;
	Delta_Y = p.y - res_down_y;

	res_down_x = p.x;
	res_down_y = p.y;

	Property.Rect.ResizeD(Delta_X,Delta_Y);
	Control.RepaintAll();
    }

    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow **) ControlList.GetItem(i);

	if (!Points->Property.Visible)
	    continue;

	r = MapToScreen(Points->Property.Rect,Points);
	r2 = MapToScreen(Property.Rect,this);

	if (r2.IsIn(r))
	    Points->OnMove(p,Status);
    }

    return true;
}

bool CWindow::OnCommand(int Handle,long int Event,CPoint p,int Key,
			   int Status)
{
    return true;
}

bool CWindow::OnSignal(int Handle,long int Event,void *Data[5])
{
    return true;
}


/***********************************************************************/

int CWindow::AddControl(CWindow *w)
{
    if (!w) 
	return 0;

    if (CurrentControl == 0) CurrentControl = ControlCount;
    w->Property.Handle = ControlCount++;
    w->Property.Parent = this;

    ControlList.AddTop(&w);


    if (IsCreated) {
	w->OnPreCreate();
	w->OnCreate();
    }

    return w->Property.Handle;
}

bool CWindow::DeleteControl(int Handle)
{
    CWindow *Points;
    int i;

    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow**) ControlList.GetItem(i);
	if (Points->Property.Handle == Handle) {
	    ControlList.Delete(i);

	    if (CurrentControl==Handle) Points->OnLoseFocus();
	    Points->OnClose();
	    delete Points;

	    ControlCount--;
	    if (ControlCount>0) {
		Points = *(CWindow **) ControlList.GetItem(0);
		Points->OnGetFocus();
		CurrentControl = Points->Property.Handle;
	    }
	    else 
		CurrentControl = -1;

	    return true;
	}
    }

    return false;
}

CWindow *CWindow::GetControl(int Handle)
{
    CWindow *Points;
    int i;

    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow**) ControlList.GetItem(i);
	if (Points->Property.Handle == Handle)
	    return Points;
    }

    return NULL;
}

int CWindow::ControlFromPoint(CPoint p)
{
    int i;
    CWindow *Points;
    CWindow *Current;
    CRect r;
    CRect r2;


    if (CurrentControl != -1) {
	Current = GetControl(CurrentControl);

	if (Current == NULL) {
	    CurrentControl = -1;
	    return CurrentControl;
	}

	r = MapToScreen(Current->Property.Rect,Current);
	if (r.IsIn(p) && Current->Property.Visible)
	    return CurrentControl;
	else
	    Current->OnLoseFocus();
    }

    for (i=0;i<ControlList.GetTotal();i++) {
	Points = *(CWindow **) ControlList.GetItem(i);

	if (!Points->Property.Visible)
	    continue;

	r = MapToScreen(Points->Property.Rect,Points);
	r2 = MapToScreen(Property.Rect,this);

	if (r.IsIn(p) && r2.IsIn(r)) {
	    CurrentControl = Points->Property.Handle;
	    Points->OnGetFocus();
	    return CurrentControl;
	}
    }

    CurrentControl = -1;

    return CurrentControl;
}

bool CWindow::NextControl()
{
    int i;
    CWindow *Points, *ConWindow;
    int total = ControlList.GetTotal();
    bool picknext = false;
    CRect r, r2;

    if (!total)
        return false;
    else if (CurrentControl < 0)
        picknext = true;

    r2 = MapToScreen(Property.Rect,this);

    for (i=0;i<total*2;i++) {
        Points = *(CWindow **) ControlList.GetItem(i%total);

        if (Points->Property.Handle == CurrentControl)
            picknext = true;
        else if (picknext) {
            r = MapToScreen(Points->Property.Rect,Points);

            if (Points->Property.Style & CWIN_NOTABSTOP ||
                !Points->Property.Visible ||
                !r2.IsIn(r))
                continue;

            if (CurrentControl != -1) {
                ConWindow = GetControl(CurrentControl);
                ConWindow->OnLoseFocus();
            }

            CurrentControl = Points->Property.Handle;
            Points->OnGetFocus();
            break;
        }
    }

    return true;
}

CRect CWindow::MapToScreen(CRect r,CWindow *p)
{
    int Height,Width;
    int x1,y1,x2,y2;
    int mx1,my1,mx2,my2;
    CWindow *Points;

    Width = r.GetWidth();
    Height = r.GetHeight();
    Points = p;

    r.GetCoords(x1,y1,x2,y2);

    while (Points->Property.Parent) {
	Points->Property.Parent->Property.Rect.GetCoords(mx1,my1,mx2,my2);
	x1 += mx1;
	y1 += my1;

	Points = Points->Property.Parent;
    }

    x2 = x1 + Width - 1;
    y2 = y1 + Height - 1;

    r.SetCoordsNoClip(x1,y1,x2,y2);

    return r;
}
