/*
 * bettor.c: Sample task of a betting game.
 *
 * (C) Copyright 2003, netcluster
 *
 * Syntax: bettor [betting_limit]
 *
 * Description:
 *   Two processes take turns betting. To make a bet one process calls
 *   the other with its bet. A players bet is his opponent's last bet
 *   plus a random number between 1 and 100, provided the total does
 *   not exceed the players betting limit. It it does, the player gives
 *   0 as a bet, signifying that he has lost. Player one is designated
 *   as the first bettor and initially calls the other with a bet of
 *   1. After this, the processes alternate between waiting for the
 *   next bet and playing a bet. This continues until one of them places
 *   a bet of 0.
 *
 * Written by Fakhir Ansari
 *
 * See the COPYING file for more info
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>

#include "libnc.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * USER TASKS
 * The task_main is the task entry point for all binaries.
 * The rest of the subtasks could be arbitratily named but
 * no other subtask takes any arguments or return values
 * except for task_main. The first argument (argv[0]) of
 * task main is the start symbol (i.e. "task_main"). The rest
 * are the arguments given on the command line after the
 * binary name.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#define DEFAULT_BETTING_LIMIT 1000

#define playwith 0
#define placebet 1
RPC_FUNC(void playwith(task_id player, int first, int limit))
RPC_FUNC(void placebet(int bet))

void task_main(int argc, char *argv[])
{
	task_id a;
	task_id b;
	int limit = DEFAULT_BETTING_LIMIT;

	if (argc >= 2)
		limit = atoi(argv[1]);

	printf("Loaded bettor master task successfully\n");
	printf("Setting betting limit to %d\n", limit);

	a = nc_createtask("bettor");
	if (TASK_ID_ISNULL(a))
		printf("Unable to create player 1\n");
	else
		printf("Created player 1\n");

	b = nc_createtask("bettor");
	if (TASK_ID_ISNULL(b))
		printf("Unable to create player 2\n");
	else
		printf("Created player 2\n");

	/* Call playwith of each player */
	nc_call(&a, playwith, b, 1, limit);
	nc_call(&b, playwith, a, 2, limit);
}

SUBTASK(bettor)
{
	int mybet = 1;
	int hisbet = 1;
	task_id opponent;
	int first;
	int limit;

	srand(time(NULL));

	if (nc_accept(playwith, &opponent, &first, &limit))
		nc_treturn(0);

	printf("Loaded player %d\n", first);

	if (first == 1) {
		nc_call(&opponent, placebet, mybet);
		printf("My bet: %d\n", mybet);
	}

	while (mybet > 0 && hisbet > 0) {
		if (nc_accept(placebet, &hisbet))
			nc_treturn(0);

		printf("His bet: %d\n", hisbet);

		if (hisbet > 0) {
			mybet = hisbet + 1 + rand()%100;
			if (mybet > limit)
				mybet = 0;

			nc_call(&opponent, placebet, mybet);
			printf("My bet: %d\n", mybet);
		}
	}

	if (mybet == 0)
		printf("I lost. Last bet: %d\n", hisbet);
	else
		printf("I won. Last bet: %d\n", mybet);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * LIBNC DATA
 * This section is HIDDEN. It is currently manually written but
 * would later be generated by ncpp, the netcluster preprocessor,
 * based on the above source.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* User function definitions indexed by function ID */
nc_rpcdef_t nc_rpcs[] = {
	{ "playwith", "%t%d%d", "%z" },
	{ "placebet", "%d", "%z" },
	{ NULL, NULL, NULL }
};

/* User task definitions not in any particular order */
nc_taskdef_t nc_tasks[] = {
	{ "bettor", bettor },
	{ NULL, NULL }
};
