/*
 * prime.c: Sample task to calculate prime numbers
 * (C) Copyright 2003, netcluster
 *
 * Syntax: prime num1 num2 [num3 num4 [num5 num6 ...]]
 *
 * Description:
 *   The program takes n pairs of integers ranges on the command
 *   line and then creates n processes, each of which calculate
 *   and display all prime numbers within its allocated range.
 *
 * Written by Kashif Jamal
 * Further modified by Fakhir Ansari
 *
 * See the COPYING file for more info
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>

#include "libnc.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * UTILITY FUNCTIONS
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int is_prime(int n)
{
	int i;

	for (i=2;i<n/2+1;i++) {
		if (!(n%i))
			return FALSE;
	}

	return TRUE;
}

void prime_range(int a, int b)
{
	int cnt = 0;

	for (;a<=b;a++) {
		if (is_prime(a)) {
			fprintf(stderr, "%d, ", a);
			cnt++;

			if (!(cnt%8))
				fprintf(stderr, "\n");
		}
	}

	if (cnt%8)
		fprintf(stderr, "\n");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * USER TASKS
 * The task_main is the task entry point for all binaries.
 * The rest of the subtasks could be arbitratily named but
 * no other subtask takes any arguments or return values
 * except for task_main. The first argument (argv[0]) of
 * task main is the start symbol (i.e. "task_main"). The rest
 * are the arguments given on the command line after the
 * binary name.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#define num_range 0
RPC_FUNC(void num_range(int a, int b))

void task_main(int argc, char *argv[])
{
	int i;
	task_id newtask;
	int taskcnt;

	fprintf(stderr, "prime.c: Loaded master task.\n");

	if (argc < 2) {
		fprintf(stderr, "Syntax: prime num1 num2 [num3 num4 [...]].\n");
		fprintf(stderr, "No number range specified on the command line. Exiting.\n");
		return;
	}
	else if (!(argc%2)) {
		fprintf(stderr, "Ranges must be specified in pairs. Exiting.\n");
		return;
	}

	taskcnt = 0;

	for (i=1;i<argc;i+=2) {
		taskcnt++;
		newtask = nc_createtask("prime_subtask");

		if (TASK_ID_ISNULL(newtask))
			fprintf(stderr, "Unable to create child task %d\n", taskcnt);
		else
			fprintf(stderr, "Successfully created child task %d\n", taskcnt);

		/* Call the num_range transaction of subtask */
		nc_call(&newtask, num_range, atoi(argv[i]), atoi(argv[i+1]));
	}

	/* nc_terminate(1); */
}

SUBTASK(prime_subtask)
{
	int i;
	int a,b;

	fprintf(stderr, "prime.c: Loaded subtask\n");

	if (nc_accept(num_range, &a, &b)) {
		fprintf(stderr, "Accept num_range successful: %d-%d\n", a, b);
		nc_treturn(0);
	}
	else
		fprintf(stderr, "Accept num_range failed\n");

	if (a > b)
		prime_range(b, a);
	else
		prime_range(a, b);
	sleep(2);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * LIBNC DATA
 * This section is HIDDEN. It is currently manually written but
 * would later be generated by ncpp, the netcluster preprocessor,
 * based on the above source.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* User function definitions indexed by function ID */
nc_rpcdef_t nc_rpcs[] = {
	{ "num_range", "%d%d", "%z" },
	{ NULL, NULL, NULL }
};

/* User task definitions not in any particular order */
nc_taskdef_t nc_tasks[] = {
	{ "prime_subtask", prime_subtask },
	{ NULL, NULL }
};
