/*
 * ioredir.c: Handles redirection of stardard I/O for user
 * tasks. All I/O is redirected to the master task.
 * (C) Copyright 2003, netcluster
 *
 * Written by Fakhir Ansari
 *
 * See the COPYING file for more info
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "defs.h"
#include "structs.h"
#include "log.h"
#include "msgs.h"
#include "task.h"
#include "ioredir.h"


/* Size of the redirection buffer. Amount of data that could be
 * transferred in one go, in bytes. Includes the NULL terminator. */
#define DEFAULT_IOREBUF_SIZE 512


/* redir_stdfd: Redirects the output from a child task to its
 * master. The specified descriptor maybe stdout or stderr of
 * the task. Both are treated similarly. Returns no. of bytes
 * redirected to the master task or -1 on error. */
int redir_stdfd(int fd)
{
	msghdr_t hdr;
	taskinfo_t *find;
	int nbytes;
	char data[sizeof(ioredir_t) + DEFAULT_IOREBUF_SIZE];
	ioredir_t *ioredir = (ioredir_t*)data;
	char *buffer = data + sizeof(ioredir_t);

	if (!(find = task_find_stdio(fd))) {
		yell("No task has std_io matching LOCALDATA fd<%d>", fd);
		return -1;
	}

	nbytes = read(fd, buffer, DEFAULT_IOREBUF_SIZE-1);

	if (nbytes < 0) {
		yell("Unable to read stdio redir to fd<%d>", fd);
	}
	else if (nbytes > 0) {
		buffer[nbytes] = '\0';

		hdr.sender = dconf.self_id;
		hdr.receiver = find->master.cdaemon;
		hdr.msg_type = MSG_IOREDIR_OUT;
		hdr.priority = 1;
		hdr.data_size = sizeof(ioredir_t)+nbytes+1;
		hdr.trans_id = TRANSID_NONE;

		ioredir->id = find->id;
		ioredir->master = find->master;

		if (!memcmp(&find->master.cdaemon, &dconf.self_id, sizeof(daemon_id))) {
			taskinfo_t *minf = task_find(&find->master);
			
			if (minf && minf->loader_fd >= 0) {
				msg_fd_write(&hdr, data, minf->loader_fd);
			}
		}
		else {
			msg_write(&hdr, data);
		}
	}

	return nbytes;
}
