#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

#include "defs.h"
#include "structs.h"
#include "clist.h"
#include "ctimer.h"
#include "sock.h"
#include "msgs.h"
#include "io.h"
#include "transact.h"
#include "connman.h"
#include "util.h"
#include "log.h"


/* create task request timeout in seconds */
#define CREATE_TASK_TIMEOUT 10

int ncloader_deinit();


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *              L o a d e r   M e s s a g e   H a n d l e r s
 *     a n d   T r a n s a c t i o n   F a i l u r e   H a n d l e r s
 *
 * Below are the function(s) for handling incoming messages to the loader.
 * Check the MSG_HANDLER definition in msgs.h for the parameters that
 * the message handler gets. All these are callback functions. They
 * return the new status of the message, which is MSGSTAT_DEL if it
 * is used and maybe removed from the msg queue.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* hnd_invalid: Handles invalid messages. */
MSG_HANDLER(hnd_invalid)
{
	yell("Received invalid msg: msg_type<%d> data_size<%d>. Trashed!",
			hdr->msg_type, hdr->data_size);
	return MSGSTAT_DEL;
}

/*
 * hnd_stat_create_task: Receieves the status of a task creation request.
 * If the request failed, an error code is returned. Is a transaction.
 */
MSG_HANDLER(hnd_create_task_rply)
{
	taskrply_t *rply = data;

	ntoh_taskrply(rply);

	if (rply->status == TASKRPLY_STATUS_SUCCESS) {
		do_log(LOG_INFO, "Master task successfully created.");
		/* ncloader_deinit(); */
		/* exit(EXIT_SUCCESS); */
	}
	else {
		yell("Daemon returned failure. Unable to create task.");
		ncloader_deinit();
		exit(EXIT_FAILURE);
	}

	return MSGSTAT_DEL;
}

/* hndfail_create_task_rply: Handles the timeout of a create task req. */
TRANSFAIL_HANDLER(hndfail_create_task_rply)
{
	yell("Request timed out: No response to task creation from daemon.");
	ncloader_deinit();
	exit(EXIT_FAILURE);
}

/*
 * MSG_IOREDIR_OUT: Msg sent from daemon containing the stdout/stderr
 * data of a task which is either remote or local.
 * Expects: ioredir_t + ASCIIZ_data
 * Type: Async
 */
MSG_HANDLER(hnd_ioredir_out)
{
	/* no need to cast ioredir_t - its in nbo */
	ioredir_t *iohdr = (ioredir_t*)data;
	char *iodata = ((char*)data) + sizeof(ioredir_t);

	do_log(LOG_INFO, "----> %s <---- pid<%d>", iodata, ntohl(iohdr->id.pid));

	return MSGSTAT_DEL;
}


/* Initialize the message handlers to the above functions */
msghandler_t msg_handlers[NUMBER_OF_HANDLERS] = {
	{ hnd_invalid },		/* msg_type = 0 */
	{ NULL },			/* msg_type = 1 */
	{ NULL },			/* msg_type = 2 */
	{ NULL },			/* msg_type = 3 */
	{ NULL },			/* msg_type = 4 */
	{ NULL },			/* msg_type = 5 */
	{ NULL },			/* msg_type = 6 */
	{ NULL },			/* msg_type = 7 */
	{ NULL },			/* msg_type = 8 */
	{ NULL },			/* msg_type = 9 */
	{ NULL },			/* msg_type = 10 */
	{ NULL },			/* msg_type = 11 */
	{ NULL },			/* msg_type = 12 */
	{ NULL },			/* msg_type = 13 */
	{ NULL },			/* msg_type = 14 */
	{ NULL },			/* msg_type = 15 */
	{ NULL },			/* msg_type = 16 */
	{ NULL },			/* msg_type = 17 */
	{ NULL },			/* msg_type = 18 */
	{ NULL },			/* msg_type = 19 */
	{ NULL },			/* msg_type = 20 */
	{ NULL },			/* msg_type = 21 */
	{ NULL },			/* msg_type = 22 */
	{ NULL },			/* msg_type = 23 */
	{ NULL },			/* msg_type = 24 */
	{ NULL },			/* msg_type = 25 */
	{ NULL },			/* msg_type = 26 */
	{ NULL },			/* msg_type = 27 */
	{ NULL },			/* msg_type = 28 */
	{ NULL },			/* msg_type = 29 */
	{ NULL },			/* msg_type = 30 */
	{ NULL },			/* msg_type = 31 */
	{ NULL },			/* msg_type = 32 */
	{ NULL },			/* msg_type = 33 */
	{ NULL },			/* msg_type = 34 */
	{ NULL },			/* msg_type = 35 */
	{ NULL },			/* msg_type = 36 */
	{ NULL },			/* msg_type = 37 */
	{ hnd_create_task_rply },	/* msg_type = 38 */
	{ hnd_ioredir_out },		/* msg_type = 39 */
	{ NULL },			/* msg_type = 40 */
	{ NULL },			/* msg_type = 41 */
	{ NULL },			/* msg_type = 42 */
	{ NULL },			/* msg_type = 43 */
	{ NULL },			/* msg_type = 44 */
	{ NULL },			/* msg_type = 45 */
	{ NULL },			/* msg_type = 46 */
	{ NULL },			/* msg_type = 47 */
	{ NULL },			/* msg_type = 48 */
	{ NULL }			/* msg_type = 49 */
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                             L o a d e r
 *
 * The loader sends messages to a local daemon to load a master task.
 * The following mechanisms are the same as those used in the daemon
 * so that the loader is completely extendible. Any change in the msg
 * sending or formatting method in the daemon, for example, would
 * directly be inherited by the loader even.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

daemon_id self_id;
int opt_daemon_port = DPORT;
taskreq_t opt_master_request;

/*
 * send_create_taskreq: Sends a request to the local daemon
 * requesting the creation of a master task. The function returns
 * after sending the request and the status of this request is
 * handled by the hnd_create_task_rply. Returns TRUE if the request
 * was successfully sent.
 */
int send_create_taskreq()
{
	msghdr_t hdr;
	int nmsgs;

	hdr.sender = self_id;
	hdr.receiver = self_id;
	hdr.msg_type = MSG_CREATE_MASTER;
	hdr.priority = 1;
	hdr.data_size = sizeof(taskreq_t);
	hdr.trans_id = TRANSID_NONE;

	hton_taskreq(&opt_master_request);
	return trans_write(&hdr, &opt_master_request, hndfail_create_task_rply,
			NULL, CREATE_TASK_TIMEOUT);
}

/* ncloader_init: Performs all the initialization of the loader */
int ncloader_init(int argc, char *argv[])
{
	/* Parse the command line arguments */
	if (argc == 4) {
		if (!strcmp(argv[2], "-p"))
			opt_daemon_port = atoi(argv[3]);
	}
	if (argc >= 2) {
		if (access(argv[1], X_OK)) {
			yell("Unable to locate an executable");
			return FALSE;
		}
		strcpy(opt_master_request.file_name, argv[1]);
		strcpy(opt_master_request.start_sym, "");
		TASK_ID_NULL(opt_master_request.master);
		TASK_ID_NULL(opt_master_request.parent);
		opt_master_request.priority = 1;
	}
	else {
		yell("Syntax: %s file_name [-p local_daemon_port]", argv[0]);
		return FALSE;
	}
	
	/* Initialize the open fd to zero */
	FD_ZERO(&fd_ractive);
	FD_ZERO(&fd_wactive);

	/* Set the address of the controlling daemon */
	make_self_daemon_id(&self_id, opt_daemon_port);

        /* Register trans_run to be called every second */
	ctimer_add(1, 0, trans_run, NULL);
		
	return TRUE;
}

/* ncloader_deinit: Performs the deinitialization before exit */
int ncloader_deinit()
{
	/* Close all connections */
	conn_close_all();

	/* Delete all timers */
	ctimer_remove_all();
	
	return TRUE;
}

/*
 * ncloader_idle: This function is continuously called in the idle loop.
 * It performs background tasks such as timing out connections and
 * running timers.
 */
void ncloader_idle()
{
	static time_t lasttime;
	
	if (lasttime != time(NULL)) {
		lasttime = time(NULL);

		/* Timeout the expired connections */
		conn_dec_ttl();

		/* Call the timer handler */
		ctimer_run();
	}
}

/*
 * The main control of the loader subsystem. Its tasks include listening
 * for incoming connections and data on those connections. It calls the
 * communication layer for reading the messages and then dispatches them.
 */
int ncloader_main()
{
	int i;
	struct timeval tv;
	fd_set fd_read;
	int nmsgs;
	int size;

	if (!send_create_taskreq()) {
		yell("Unable to send task creation request. Make sure the "
				"daemon port specified is correct");
		return FALSE;
	}
	
	while (1) {
		ncloader_idle();

		fd_read = fd_ractive;
		tv.tv_sec = 0;
		tv.tv_usec = 500;

		if (!select(global_max_fd+1, &fd_read, NULL, NULL, &tv))
			continue;

		for (i=0;i<=global_max_fd;i++) {
			if (!FD_ISSET(i, &fd_read))
				continue;

			switch (FDTYPE(i)) {
				case FDTYPE_MSG:
					/* data on one of connected descriptors */

					/* if nmsgs < 0 then either an error
					 * has occured or the connection has
					 * been closed. if zero then a partial
					 * msg has been read and is buffered
					 * otherwise number of msg read is
					 * returned */
					nmsgs = msg_read(i);
					conn_reset_ttl(i);

					if (nmsgs < 0) {
						conn_fd_close(i);
						yell("Request timeout. Aborting...");
						return FALSE;

					}
					else if (nmsgs > 0)
						msg_dispatch();
					break;

				default:
					yell("Unknown file des: fd<%d> type<%d>",
							i, FDTYPE(i));
					return FALSE;
			};
		}
	}

	return TRUE;
}

/* main: Program entry point */
int main(int argc, char *argv[])
{
	if (ncloader_init(argc, argv)) {
		ncloader_main();
		ncloader_deinit();
	}
	else
		yell("Initialization failed. Aborting...");

	return 0;
}
