/*
 * wtaskload.c: Another version of taskload
 * linked with libwcon user interface.
 * (C) Copyright 2003, netcluster
 *
 * Written by Fakhir Ahmed Ansari
 *
 * See the COPYING file for more info.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <new.h>

#include "defs.h"
#include "structs.h"
#include "clist.h"
#include "ctimer.h"
#include "sock.h"
#include "msgs.h"
#include "io.h"
#include "transact.h"
#include "connman.h"
#include "util.h"
#include "log.h"

#include "data.h"
#include "xdraw.h"
#include "screen.h"
#include "xinter.h"
#include "xcontrol.h"
#include "xutil.h"

#define DEFAULT_DESKTOP "../nc.desk"

int task_log(task_id *tid, char *data);
int ncloader_deinit();

/* Need to define these global objects */
CScreen MyScreen;
CDraw MyDraw;
CControl Control;

/* List of open task windows */
clist *task_winlist = NULL;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                 L I B W C O N   I n t e r f a c e
 *
 * This section contains the interface dependent functions and classes
 * required to build the libwcon user interface. Only the interface
 * is dependent on C++.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


/* The status windows. Registers status messages and the main menu.
 * The window could be resized by dragging the lower right corner of
 * the window frame. Just one instance is created, on startup. */
class WinStatus : public CWindow {
public:
    virtual bool OnPreCreate();
    virtual bool OnMove(CPoint p,int Status);

    bool Log(char *msg);

    /* arranges everything according to window size */
    bool ArrangeControls();
private:
    /* control handles */
    int han_text;
};

/* initialize the global window handle */
WinStatus *win_status = NULL;


bool WinStatus::OnPreCreate()
{
    CWindow::OnPreCreate();

    Property.SetStyle(CWIN_BORDER|CWIN_RESIZE|CWIN_MOVE|
                      CWIN_TABCYCLE|CWIN_TITLE);
    Property.SetCoords(3, MAX_Y-17, MAX_X-40, MAX_Y-3);
    Property.SetTitle("Status");

    CTextBox *text = new CTextBox;
    text->Property.SetStyle(CWIN_BORDER);
    text->Property.Rect.SetMinWidth(20);
    text->Property.Rect.SetMinHeight(8);
    text->Property.SetCoords(1, 1, 5, 5);
    text->Property.Response = this;
    han_text = AddControl(text);

    ArrangeControls();

    return true;
}

bool WinStatus::OnMove(CPoint p,int Status)
{
    CWindow::OnMove(p, Status);

    if (IsResizing) {
        ArrangeControls();
        OnPaint();
    }

    return true;
}

bool WinStatus::Log(char *msg)
{
    CTextBox *tb = (CTextBox*) GetControl(han_text);
    tb->CatText(msg);
    Control.RepaintAll();

    return true;
}

bool WinStatus::ArrangeControls()
{
    int w = Property.Rect.GetWidth();
    int h = Property.Rect.GetHeight();

    CWindow *conWin;

    CTextBox *tb = (CTextBox*) GetControl(han_text);
    tb->Property.SetCoords(2, 1, w-3, h-2);
    tb->ReAlign();

    return true;
}

class WinTask : public WinStatus {
public:
    WinTask(task_id *id) { tid = *id; }
    virtual bool OnPreCreate();
    virtual bool OnClose();

    /* Identifier of this task */
    task_id tid;
    static int pos;
};

int WinTask::pos = 2;

bool WinTask::OnPreCreate()
{
    WinStatus::OnPreCreate();

    Property.SetStyle(CWIN_BORDER|CWIN_RESIZE|CWIN_MOVE|CWIN_CLOSEB|
                      CWIN_TABCYCLE|CWIN_TITLE);
    Property.SetCoords(pos*2, pos, pos*2+60, pos+9);

    pos+=3;
    if (pos+9 >= MAX_Y) pos = 2;

    char title[MAX_LINE_SIZE];
    sprintf(title, "%s:%d", naddr_to_str(&tid.cdaemon), ntohl(tid.pid));
    Property.SetTitle(title);

    ArrangeControls();

    return true;
}

bool WinTask::OnClose()
{
	WinTask *win_task;
	clist *cur = task_winlist;

	while (cur) {
		win_task = (WinTask*)cur->data;
		if (!memcmp(&win_task->tid, &tid, sizeof(task_id))) {
			task_winlist = clist_remove_link(task_winlist, cur);
			break;
		}
		cur = cur->next;
	}

	return true;
}

/* The main menu opened at startup */
class WinMainMenu : public CWindow {
public:
    virtual bool OnPreCreate();
    virtual bool OnCommand(int Handle,long int Event,CPoint p,int Key,
                           int Status);

private:
    int han_status;
    int han_term;
    int han_restart;
    int han_quit;
};

bool WinMainMenu::OnPreCreate()
{
    CWindow::OnPreCreate();

    Property.SetStyle(CWIN_BORDER|CWIN_MOVE|CWIN_TABCYCLE);
    Property.SetCoords(MAX_X-37, 0, MAX_X-1, 3);

    CButton *but = new CButton;
    but->Property.SetCoords(2, 1, 1, 1);
    but->Property.SetTitle("Status");
    but->Property.Response = this;
    han_status = AddControl(but);

    but = new CButton;
    but->Property.SetCoords(11, 1, 1, 1);
    but->Property.SetTitle("Abort");
    but->Property.Response = this;
    han_term = AddControl(but);

    but = new CButton;
    but->Property.SetCoords(19, 1, 1, 1);
    but->Property.SetTitle("Restart"); 
    but->Property.Response = this;
    han_restart = AddControl(but);

    but = new CButton;
    but->Property.SetCoords(29, 1, 1, 1);
    but->Property.SetTitle("Quit");
    but->Property.Response = this;
    han_quit = AddControl(but);

    /* create all 3 windows */
    win_status = new WinStatus;
    Control.CreateWindow(win_status);

    return true;
}

bool WinMainMenu::OnCommand(int Handle,long int Event,CPoint p,int Key,
                            int Status)
{
    if (Handle == han_quit) {
        ncloader_deinit();
        Control.Exit();
    }
    else if (Handle == han_status) {
        if (win_status) {
            win_status->Property.Visible = !win_status->Property.Visible;
            Control.RepaintAll();
        }
    }
    else if (Handle == han_term) {
        do_log(LOG_INFO, "This feature is not currently supported.");
    }
    else if (Handle == han_restart) {
        do_log(LOG_INFO, "This feature is not currently supported.");
	task_id tid;
	CWindow *b = (CWindow*) new WinTask(&tid);
	Control.CreateWindow(b);
    }

    return true;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *              L o a d e r   M e s s a g e   H a n d l e r s
 *     a n d   T r a n s a c t i o n   F a i l u r e   H a n d l e r s
 *
 * Below are the function(s) for handling incoming messages to the loader.
 * Check the MSG_HANDLER definition in msgs.h for the parameters that
 * the message handler gets. All these are callback functions. They
 * return the new status of the message, which is MSGSTAT_DEL if it
 * is used and maybe removed from the msg queue.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* create task request timeout in seconds */
#define CREATE_TASK_TIMEOUT 45


/* hnd_invalid: Handles invalid messages. */
MSG_HANDLER(hnd_invalid)
{
	yell("Received invalid msg: msg_type<%d> data_size<%d>. Trashed!",
			hdr->msg_type, hdr->data_size);
	return MSGSTAT_DEL;
}

/*
 * hnd_stat_create_task: Receieves the status of a task creation request.
 * If the request failed, an error code is returned. Is a transaction.
 */
MSG_HANDLER(hnd_create_task_rply)
{
	taskrply_t *rply = (taskrply_t*)data;

	ntoh_taskrply(rply);

	if (rply->status == TASKRPLY_STATUS_SUCCESS) {
		do_log(LOG_INFO, "Master task successfully created.");
	}
	else {
		yell("Daemon returned failure. Unable to create task.");
		ncloader_deinit();
		exit(EXIT_FAILURE);
	}

	return MSGSTAT_DEL;
}

/* hndfail_create_task_rply: Handles the timeout of a create task req. */
TRANSFAIL_HANDLER(hndfail_create_task_rply)
{
	yell("Request timed out: No response to task creation from daemon.");
}

/*
 * MSG_IOREDIR_OUT: Msg sent from daemon containing the stdout/stderr
 * data of a task which is either remote or local.
 * Expects: ioredir_t + ASCIIZ_data
 * Type: Async
 */
MSG_HANDLER(hnd_ioredir_out)
{
	/* no need to cast ioredir_t - its in nbo */
	ioredir_t *iohdr = (ioredir_t*)data;
	char *iodata = ((char*)data) + sizeof(ioredir_t);

	task_log(&iohdr->id, iodata);

	return MSGSTAT_DEL;
}


/* Initialize the message handlers to the above functions */
msghandler_t msg_handlers[NUMBER_OF_HANDLERS] = {
	{ hnd_invalid },		/* msg_type = 0 */
	{ NULL },			/* msg_type = 1 */
	{ NULL },			/* msg_type = 2 */
	{ NULL },			/* msg_type = 3 */
	{ NULL },			/* msg_type = 4 */
	{ NULL },			/* msg_type = 5 */
	{ NULL },			/* msg_type = 6 */
	{ NULL },			/* msg_type = 7 */
	{ NULL },			/* msg_type = 8 */
	{ NULL },			/* msg_type = 9 */
	{ NULL },			/* msg_type = 10 */
	{ NULL },			/* msg_type = 11 */
	{ NULL },			/* msg_type = 12 */
	{ NULL },			/* msg_type = 13 */
	{ NULL },			/* msg_type = 14 */
	{ NULL },			/* msg_type = 15 */
	{ NULL },			/* msg_type = 16 */
	{ NULL },			/* msg_type = 17 */
	{ NULL },			/* msg_type = 18 */
	{ NULL },			/* msg_type = 19 */
	{ NULL },			/* msg_type = 20 */
	{ NULL },			/* msg_type = 21 */
	{ NULL },			/* msg_type = 22 */
	{ NULL },			/* msg_type = 23 */
	{ NULL },			/* msg_type = 24 */
	{ NULL },			/* msg_type = 25 */
	{ NULL },			/* msg_type = 26 */
	{ NULL },			/* msg_type = 27 */
	{ NULL },			/* msg_type = 28 */
	{ NULL },			/* msg_type = 29 */
	{ NULL },			/* msg_type = 30 */
	{ NULL },			/* msg_type = 31 */
	{ NULL },			/* msg_type = 32 */
	{ NULL },			/* msg_type = 33 */
	{ NULL },			/* msg_type = 34 */
	{ NULL },			/* msg_type = 35 */
	{ NULL },			/* msg_type = 36 */
	{ NULL },			/* msg_type = 37 */
	{ hnd_create_task_rply },	/* msg_type = 38 */
	{ hnd_ioredir_out },		/* msg_type = 39 */
	{ NULL },			/* msg_type = 40 */
	{ NULL },			/* msg_type = 41 */
	{ NULL },			/* msg_type = 42 */
	{ NULL },			/* msg_type = 43 */
	{ NULL },			/* msg_type = 44 */
	{ NULL },			/* msg_type = 45 */
	{ NULL },			/* msg_type = 46 */
	{ NULL },			/* msg_type = 47 */
	{ NULL },			/* msg_type = 48 */
	{ NULL }			/* msg_type = 49 */
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                             L o a d e r
 *
 * The loader sends messages to a local daemon to load a master task.
 * The following mechanisms are the same as those used in the daemon
 * so that the loader is completely extendible. Any change in the msg
 * sending or formatting method in the daemon, for example, would
 * directly be inherited by the loader even.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

daemon_id self_id;
int opt_daemon_port = DPORT;
taskreq_t opt_master_request;
int opt_argc = 0;
char **opt_argv = NULL;

/*
 * send_create_taskreq: Sends a request to the local daemon
 * requesting the creation of a master task. The function returns
 * after sending the request and the status of this request is
 * handled by the hnd_create_task_rply. Returns TRUE if the request
 * was successfully sent.
 */
int send_create_taskreq()
{
	msghdr_t hdr;
	int nmsgs;
	char msgdata[sizeof(taskreq_t)+sizeof(taskarg_t)+MAX_ARGV_SIZE];
	taskreq_t *treq = (taskreq_t*) msgdata;
	taskarg_t *targ = (taskarg_t*) ((char*)msgdata+sizeof(taskreq_t));
	char *targbuf = msgdata+sizeof(taskreq_t)+sizeof(taskarg_t);
	int targlen = 0;

	targ->argc = opt_argc;

	if (opt_argc) {
		targlen = put_argv(targbuf, MAX_ARGV_SIZE, opt_argc, opt_argv);

		if (targlen < 0) {
			yell("Command line arguments of task too large");
			return FALSE;
		}
	}

	hdr.sender = self_id;
	hdr.receiver = self_id;
	hdr.msg_type = MSG_CREATE_MASTER;
	hdr.priority = 1;
	hdr.data_size = sizeof(taskreq_t)+sizeof(taskarg_t)+targlen;
	hdr.trans_id = TRANSID_NONE;

	*treq = opt_master_request;

	hton_taskreq(treq);
	hton_taskarg(targ);
	return trans_write(&hdr, msgdata, hndfail_create_task_rply,
			NULL, CREATE_TASK_TIMEOUT);
}

/*
 * task_log: Logs the stdout data from a task to a window. The
 * data must be in ASCIIZ format.
 */
int task_log(task_id *tid, char *data)
{
	WinTask *win_task;
	clist *cur = task_winlist;

	while (cur) {
		win_task = (WinTask*)cur->data;
		if (!memcmp(&win_task->tid, tid, sizeof(task_id)))
			break;
		cur = cur->next;
	}

	if (!cur) {
		win_task = new WinTask(tid);
		Control.CreateWindow(win_task);
		task_winlist = clist_append(task_winlist, win_task);
	}

	win_task->Log(data);
	/* win_task->Log("\n"); */
	Control.RepaintAll();
	MyScreen.Refresh();

	return TRUE;
}

/* ncloader_init: Performs all the initialization of the loader */
int ncloader_init(int argc, char *argv[])
{
	int arg_fname = 1;

	/* Parse the command line arguments */
	if (argc >= 4) {
		if (!strcmp(argv[1], "-p")) {
			opt_daemon_port = atoi(argv[2]);
			arg_fname += 2;
		}
	}
	if (argc >= 2) {
		if (access(argv[arg_fname], X_OK)) {
			MyScreen.Print("Unable to locate an executable\n");
			return FALSE;
		}

		*opt_master_request.file_name = '\0';
		if (*argv[arg_fname] != DIR_SEPARATOR) {
			char dirsep[2] = { DIR_SEPARATOR, 0 };

			getcwd(opt_master_request.file_name,
					sizeof(opt_master_request.file_name));
			strcat(opt_master_request.file_name, dirsep);
		}
		strcat(opt_master_request.file_name, argv[arg_fname]);

		strcpy(opt_master_request.start_sym, "");
		TASK_ID_NULL(opt_master_request.master);
		TASK_ID_NULL(opt_master_request.parent);
		opt_master_request.priority = 1;

		if (arg_fname+1 < argc) {
			opt_argc = argc - (arg_fname+1);
			opt_argv = &argv[arg_fname+1];
		}
	}
	else {
		MyScreen.Print("Syntax: %s [-p local_daemon_port] file_name [arg1 [...]]\n", argv[0]);
		return FALSE;
	}
	
	/* Initialize the open fd to zero */
	FD_ZERO(&fd_ractive);
	FD_ZERO(&fd_wactive);

	/* Set the address of the controlling daemon */
	make_self_daemon_id(&self_id, opt_daemon_port);

        /* Register trans_run to be called every second */
	ctimer_add(1, 0, trans_run, NULL);

	if (!send_create_taskreq()) {
		MyScreen.Print("Unable to send task creation request. Make sure the "
				"daemon port specified is correct\n");
		return FALSE;
	}

	return TRUE;
}

/* ncloader_deinit: Performs the deinitialization before exit */
int ncloader_deinit()
{
	/* Close all connections */
	conn_close_all();

	/* Delete all timers */
	ctimer_remove_all();

	return TRUE;
}

/*
 * ncloader_idle: This function is continuously called in the idle loop.
 * It performs background tasks such as timing out connections and
 * running timers.
 */
void ncloader_idle()
{
	static time_t lasttime;
	
	if (lasttime != time(NULL)) {
		lasttime = time(NULL);

		/* Timeout the expired connections */
		conn_dec_ttl();

		/* Call the timer handler */
		ctimer_run();
	}
}

/*
 * The main control of the loader subsystem. Its tasks include listening
 * for incoming connections and data on those connections. It calls the
 * communication layer for reading the messages and then dispatches them.
 */
int ncloader_main()
{
	static int i;
	static struct timeval tv;
	static fd_set fd_read;
	static int nmsgs;
	static int size;

	ncloader_idle();

	fd_read = fd_ractive;
	tv.tv_sec = 0;
	tv.tv_usec = 0;

	if (select(global_max_fd+1, &fd_read, NULL, NULL, &tv)) {
		for (i=0;i<=global_max_fd;i++) {
			if (!FD_ISSET(i, &fd_read))
				continue;

			switch (FDTYPE(i)) {
				case FDTYPE_MSG:
					/* data on one of connected descriptors */

					/* if nmsgs < 0 then either an error
					 * has occured or the connection has
					 * been closed. if zero then a partial
					 * msg has been read and is buffered
					 * otherwise number of msg read is
					 * returned */
					nmsgs = msg_read(i);
					conn_reset_ttl(i);

					if (nmsgs < 0) {
						conn_fd_close(i);
						yell("Request timeout. Aborting...");
						return FALSE;

					}
					else if (nmsgs > 0)
						msg_dispatch();
					break;

				default:
					yell("Unknown file des: fd<%d> type<%d>",
							i, FDTYPE(i));
					return FALSE;
			}
		}
	}

	return TRUE;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                    M a i n   C o n t r o l
 *
 * This is the main control that runs both the interface and the
 * loader. The interface is called in the idle loop of the loader.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* The Controller class calls this function in the idle loop. Could
 * perform any application dependent task in here. */
void re_idle_handler()
{
	if (!ncloader_main()) {
		ncloader_deinit();
		Control.Exit();
	}

	return;
}

/* called for every key press. Should return true if the key is handled
 * and false if not, to let the Control handle it */
bool re_key_handler(int Key, int Status)
{
	/* return if a dialog is active */
	CWindow *curwin = Control.GetActiveWindow();
	if (curwin && curwin->Property.Style&CWIN_DIALOG)
		return false;

	/* switching to next window with F1 */
	if (Key == KEY_F(1)) {
		Control.SwitchWindow(-1);
		return true;
	}

	/* handle the application dependent keys */
	/* switch (Key) {} */

	return false;
}

void re_logger(int type, char *fmt, ...)
{
	va_list args;

	/* Ignore if this type of msgs not being logged */
	if (!(get_log_types() & type))
		return;

	va_start(args, fmt);
	vsnprintf(log_buffer, MAX_LOG_SIZE-1, fmt, args);
	va_end(args);

	log_buffer[MAX_LOG_SIZE-1] = '\0';
	strcat(log_buffer, "\n");

	if (win_status)
		win_status->Log(log_buffer);

	Control.RepaintAll();
	MyScreen.Refresh();
}

void re_logger_yell(char *fmt, ...)
{
	va_list args;

	/* Ignore if this type of msgs not being logged */
	if (!(get_log_types() & LOG_ERR))
		return;

	va_start(args, fmt);
	vsnprintf(log_buffer, MAX_LOG_SIZE-1, fmt, args);
	va_end(args);

	log_buffer[MAX_LOG_SIZE-1] = '\0';
	strcat(log_buffer, "\n");

	if (win_status)
		win_status->Log(log_buffer);

	Control.RepaintAll();
	MyScreen.Refresh();
}


/* Handler called if 'new' fails */
void custom_handle()
{
	fprintf(stderr, "Error: New enough memory. Aborting...\n");
	exit(EXIT_FAILURE);
}


int main(int argc, char *argv[])
{
	set_new_handler(custom_handle);

	if (!ncloader_init(argc, argv)) {
		MyScreen.Print("Initialization failed. Aborting...\n");
		MyScreen.Refresh();
		return 1;
	}
	
	if (!access(DEFAULT_DESKTOP, R_OK))
		Control.SetDesktop(DEFAULT_DESKTOP);
		
	/* Set the colors used in all the controls */
	THEME_LIGHT = COLOR_WHITE + H_INTENSITY;
	THEME_NORM = COLOR_WHITE;
	THEME_DARK = COLOR_BLACK + H_INTENSITY;

	/* Create a status window initially */
	WinMainMenu *win_mainmenu = new WinMainMenu;
	Control.CreateWindow(win_mainmenu);

	do_log = re_logger;
	yell = re_logger_yell;
	do_log(LOG_INFO, "Initialization successful");
	
	Control.RepaintAll();
	Control.Run();

	return 0;
}
