/*
 * vasmain.c: The main vas file.
 * (C) Copyright 2003, netcluster
 *
 * Written by Syed Ghaus Ali
 *
 * See the COPYING file for more info
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

#include "defs.h"
#include "clist.h"
#include "chtable.h"
#include "sock.h"
#include "msgs.h"
#include "io.h"
#include "connman.h"
#include "util.h"
#include "conf.h"
#include "log.h"
#include "vas.h"
#include "transact.h"



//#define TESTPORT 1029
int TESTPORT;
int option;
daemon_id vas;
daemon_id d;

/* VAS Interface functions  START */

/* register_daemon: To tell the VAS that it wants to be a 
 * part of the distributed computing system. This daemon id
 * is sent to VAS to be added in the VAS addresses */
int register_daemon( daemon_id  id)
{
	msghdr_t msg;
	msg.sender = id;
	msg.receiver = vas;
	msg.msg_type = MSG_REG_DAEMON;
	msg.priority = 1;
	msg.data_size = 0;
	msg.trans_id = TRANSID_NONE;
	if (!msg_write(&msg, NULL))
		fprintf(stderr, "ERROR: Unable to send message\n");
	return MSGSTAT_DEL;
}

/* unregister_daemon: To tell the VAS that it wants to be  
 * removed from the distributed computing system. This daemon 
 * is sent to VAS to be removed from the VAS addresses */
int unregister_daemon( daemon_id id)
{
	msghdr_t msg;
	msg.sender = id;
	msg.receiver = vas;
	msg.msg_type = MSG_UNREG_DAEMON;
	msg.priority = 1;
	msg.data_size = 0;	
	msg.trans_id = TRANSID_NONE;
	msg_write(&msg, NULL);
	return MSGSTAT_DEL;
}

/* has_registered: For the given daemon id, msg will
 * be sent to vas to confirm if this id exists there */
int has_registered( daemon_id  id )
{
	msghdr_t msg;
	msg.sender = id;
	msg.receiver = vas;
	msg.msg_type = MSG_HAS_REG_DAEMON;
	msg.priority = 1;
	msg.data_size = 0;
	msg.trans_id = TRANSID_NONE;
	msg_write(&msg, NULL);
	return 0;
}

/* set_load: for the given daemon id, that load is send 
 * to vas to be updated */
void set_load( daemon_id id, long load)
{
	char * data;
	msghdr_t msg;
	msg.sender = id;
	msg.receiver = vas;
	msg.msg_type = MSG_SET_LOAD;
	msg.priority = 1;
	msg.data_size = 4;
	data = malloc(4);
	msg.trans_id = TRANSID_NONE;
	memcpy(data,&load, 4);
	msg_write(&msg, data);
}

/* get_load: This is send a msg to vas to get the load 
 * corresponding to this daemon id at vas.*/
void get_load( daemon_id id )
{
	msghdr_t msg;
	msg.sender = id;
	msg.receiver = vas;
	msg.msg_type = MSG_GET_LOAD;
	msg.priority = 1;
	msg.data_size = 0;
	msg.trans_id = TRANSID_NONE;
	msg_write(&msg, NULL);
}

/*get_idle: this will request vas to send the daemonid 
 * that has the minimin load. */
void get_idle(daemon_id id)
{
	msghdr_t msg;
	msg.sender = id;
	msg.receiver = vas;
	msg.msg_type = MSG_GET_DAEMON_MIN_LOAD;
	msg.priority = 1;
	msg.data_size = 0;
	msg.trans_id = TRANSID_NONE;
	msg_write(&msg, NULL);
}

/* VAS Interface functions  END */




clist *daemons_gone=NULL;
chashtable *vas_table=NULL;


/* 
 * init_all: Performs all the initialization of vas
 */
int init_all()
{
        /*init_conf();*/
	FD_ZERO(&fd_ractive);
	FD_ZERO(&fd_wactive);
	return TRUE;
}

/* 
 * deinit_all: Performs the deinitialization of the vas before exit 
 */
int deinit_all()
{
        /*deinit_conf();*/
	return TRUE;
}

/*
 * ncdaemon_idle: This function is continuously called in the idle loop.
 * It performs background tasks such as timing out connections and
 * running timers.
 */
void ncvas_idle()
{
        static time_t lasttime;
	
	if (lasttime != time(NULL)) {
		lasttime = time(NULL);

		
		conn_dec_ttl();
	}
}


/*
 * ncvas_main: The main control of the vas. Its tasks include listening
 * for incoming messages and and handling them accordingly by calling
 * msg_dispatch function
 */
int ncvas_main()
{
	int i;
	int flag;
	struct timeval tv;
	fd_set fd_read;
	struct sockaddr_in clientaddr;
	int sock;
	int nmsgs;
	int size;
	int new;
       
	if ((sock = make_listen_socket(TESTPORT)) < 0) {
		fprintf(stderr, "Unable to listen on port %d. Aborting...\n",TESTPORT);
		return FALSE;
	}
	
	/* Dont register as connection coz its only for listening */
	fd_open(sock, FDTYPE_LISTEN);
	flag = 0;
	while (1) {
	  if(!flag){
	    flag = 1;
	    if(option == 3){
	      printf("Confirming registation...\n");
	      has_registered(d);
	    }
	    else if(option == 5){
	      printf("Attempting to Get Load...\n");
	      get_load(d);
	    }
	    else if(option == 6){
	      printf("Attempting to get idle\n");
	      get_idle(d);
	    }
	  }
		ncvas_idle();

		fd_read = fd_ractive;
		tv.tv_sec = 0;
		tv.tv_usec = 500;

		if (!select(global_max_fd+1, &fd_read, NULL, NULL, &tv))
			continue;

		for (i=0;i<=global_max_fd;i++) {
			if (!FD_ISSET(i, &fd_read))
				continue;

			switch (FDTYPE(i)) {
				case FDTYPE_LISTEN:
					/* new connection */
					if (i != sock)
						continue;

					size = sizeof(clientaddr);
					new = accept(sock, &clientaddr, &size);
					if (new < 0) {
						yell("Unable to accept:%s", strerror(errno));
						continue;
					}

					conn_fd_open(new);

					do_log(LOG_NOTICE,"Connect from addr<%s:%d> sock<%d>",
							inet_ntoa(clientaddr.sin_addr),
							ntohs(clientaddr.sin_port), new);
					break;

				case FDTYPE_MSG:
					/* data on one of connected descriptors */

					/* if nmsgs < 0 then either an error
					 * has occured or the connection has
					 * been closed. if zero then a partial
					 * msg has been read and is buffered
					 * otherwise number of msg read is
					 * returned */
					nmsgs = msg_read(i);
					conn_reset_ttl(i);

					if (nmsgs < 0) {
						conn_fd_close(i);
						do_log(LOG_NOTICE,"Session closed: sock<%d>",i);

					}
					else if (nmsgs > 0)
						msg_dispatch();
					break;

				default:
					do_log(LOG_ERR, "Unknown file des: fd<%d> type<%d>",
							i, FDTYPE(i));
			};
		}
	}
	
	return TRUE;
}






/* main: Program entry point */
int main()
{
	int load;
	msghdr_t msg;
	
#ifdef NCDAEMON
	printf("VAS TEST PROGRAM\n");
#else
	printf("WRONG");
#endif
	printf("Enter the port for this daemon...\n");
	scanf("%d", &TESTPORT);
	init_all();
	make_daemon_id(&d,"127.0.0.1",TESTPORT );  
	make_daemon_id(&vas,"127.0.0.1",8888);
	
	printf("Register         --- 1 \n");
	printf("UnRegister       --- 2 \n");
	printf("Confirm Register --- 3 \n");
	printf("Set load         --- 4 \n");
	printf("Get load         --- 5 \n");
	printf("Get Idle         --- 6 \n");
	printf("Verify list      --- 7 \n");
	printf("Show GOne Daemons--- 8 \n");
	printf("Quit             --- 9 \n");

	scanf("%d", &option);
	if(option == 1){
	  printf("Attempting to register...\n");
	  register_daemon(d);
	}
	else if(option == 2){
	  printf("Attempting to Unregister...\n");
	  unregister_daemon(d);
	}
	else if(option == 4){
	  printf("Attempting to set load... enter load\n");
	  scanf("%d", &load );
	  set_load(d,load);  
	}
	else if(option == 7){
	  printf("Attempting to verify m list\n");
	  msg.sender = d;
	  msg.receiver = vas;
	  msg.msg_type = MSG_VERIFYMLIST;
	  msg.priority = 1;
	  msg.data_size = 0;
	  msg.trans_id = TRANSID_NONE;
	  msg_write(&msg, NULL);
	}
	else if(option == 8){
	  printf("Requesting to show daemons (left) on vas...\n");
	  msg.sender = d;
	  msg.receiver = vas;
	  msg.msg_type = MSG_SHOW_GONE_DAEMONS;
	  msg.priority = 1;
	  msg.data_size = 0;
	  msg.trans_id = TRANSID_NONE;
	  msg_write(&msg, NULL);
	}
	
	ncvas_main();
	deinit_all();
	return 0;
}
